#!/usr/bin/env python

import re, os, urllib, sys
plugin_re = re.compile(r'<a class="file" title="View File" href=".*?">([-\w]+.py)</a>', re.DOTALL|re.MULTILINE)

if len(sys.argv) != 2:
    print "usage: python pluginupdater.py [exaile_version]"
    sys.exit(0)
version = sys.argv[1]
data = urllib.urlopen('http://exaile.org/trac/browser/plugins/%s' %
    version).read()

plugins = plugin_re.findall(data)


print "Updating plugin list"
for plugin in plugins:
    data = \
        urllib.urlopen('http://www.exaile.org/plugins/plugins.py?version=%s&plugin=%s'
        % (version, plugin)).read()

    plugindir = '/home/%s/.exaile/plugins' % os.getlogin()
    if not os.path.isdir(plugindir):
        os.mkdir(plugindir, 0777)

    print "Writing %s" % plugin
    h = open('/home/%s/.exaile/plugins/%s' % (os.getlogin(), plugin), 'w')
    h.write(data)
    h.close()

print "Done.\n"



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Code based on Ingelrest FranÃ§ois' "im status" plugin

import dbus, gobject, gtk, os
from gettext import gettext as _
import xl.plugins as plugins

import sys

# Plugin description
PLUGIN_NAME        = _('Current song')
PLUGIN_ICON        = None
PLUGIN_ENABLED     = False
PLUGIN_AUTHORS     = ['Jorge VillaseÃ±or <salinasv@gmail.com']
PLUGIN_VERSION     = '0.1'
PLUGIN_DESCRIPTION = _(r"""Set the current playing song in the Pidgin im client using the tune status (only the supported prpl)""")

# Constants
DEFAULT_UPDATE_ON_PAUSE = True

# Global Variables
mHandlers    = plugins.SignalContainer()
mCurrentTitle = ''

##############################################################################

class Pidgin :

    def __init__(self, dbusInterface) :
        """
            Constructor
        """
        self.purple = dbusInterface

    def listAccounts(self) :
        """
            Purple merges all accounts, so we return a default one
            Each account is associated with:
                * A boolean -> True if the status of this account was changed on the previous track change
        """
        return {'GenericAccount' : False}
    
    def setStatus(self, status, attr, value):
        # this doesn't actually work, for some reason the getter always return ""
        if self.purple.PurpleStatusGetAttrString(status, attr) != value:
            self.purple.PurpleStatusSetAttrString(status, attr, value)
            return True
        return False

    def setTune(self, artist, title, album) :
        """
            Change the tune status
            Return True if the message is successfully updated
        """
        current = self.purple.PurpleSavedstatusGetCurrent()
        accounts = self.purple.PurpleAccountsGetAll()

        for account in accounts:
            if self.purple.PurpleAccountIsConnected(account) != True:
                continue
            p = self.purple.PurpleAccountGetPresence(account)
            status = self.purple.PurplePresenceGetStatus(p, "tune")

            if status != 0:
                updated = False
                if len(title) + len(artist) + len(album) == 0:
                    if self.purple.PurpleStatusIsActive(status):
                        self.purple.PurpleStatusSetActive(status, False)
                else:
                    self.purple.PurpleStatusSetActive(status, True)
                    updated |= self.setStatus(status, "tune_title", title);
                    updated |= self.setStatus(status, "tune_artist", artist);
                    updated |= self.setStatus(status, "tune_album", album);
                if updated:
                    active = self.purple.PurplePresenceGetActiveStatus(p)
                    self.purple.PurplePrplChangeAccountStatus(account, active, status)

        return True

##############################################################################

def updateStatus() :
    """
        Information about the current track has changed
    """
    global mCurrentTrack
    global mCurrentTitle
    mCurrentTrack = APP.player.current

    title = unicode(getattr(mCurrentTrack, 'title'))
    if title == mCurrentTitle:
        return
    else :
        mCurrentTitle = title

    artist = unicode(getattr(mCurrentTrack, 'artist'))
    album = unicode(getattr(mCurrentTrack, 'album'))

    if APP.player.is_paused() :
        client.setTune("","","")
    else :
        client.setTune(artist, title, album)

def stop_track(exaile, track) :
    """
        Called when Exaile quits or when playback stops
    """
    # Stop event is sent also when a track is finished, even if Exaile is going to play the next one
    # Since we don't want to change the status between each track, we add a delay of 1/2 sec. before doing anything
    gobject.timeout_add(500, onStopTimer)


def onStopTimer() :
    """
        Called 1/2 sec. after the reception of a stop event
    """
    if not APP.player.is_playing() :
        client.setTune("","","")
    else :
        updateStatus()
    return False


def initialize(self = None):
    """
        Called when the plugin is enabled
    """

    global client
    obj = dbus.SessionBus().get_object('im.pidgin.purple.PurpleService',
                                   '/im/pidgin/purple/PurpleObject')
    purple = dbus.Interface(obj, "im.pidgin.purple.PurpleInterface")

    client = Pidgin(purple)

    # Add the handlers
    mHandlers.connect(APP.player, 'stop-track', stop_track)
    mHandlers. connect(APP.player, 'pause-toggled', lambda exaile, track : updateStatus())

    return True

def destroy() :
    """
        Called when the plugin is disabled
    """
    mHandlers.disconnect_all()



# Copyright (C) 2006 Adam Olsen 
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 1, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

import os, re
from ConfigParser import SafeConfigParser
import ConfigParser, gobject
from xl import common, xlmisc

# ok, I know this isn't secure, but it's better than having passwords in
# plaintext in the configuration file.
XOR_KEY = "You're not drunk if you can lie on the floor without hanging on"

"""
    This module provides for easy parsing of configuration files
"""

class XlConfigParser(SafeConfigParser):
    """
        Allows for easy parsing of configuration files
    """
    def __init__(self, loc):
        """
            Expects the location of a file.  Opens the file, and parses it
            to populate the dictionary
        """
        SafeConfigParser.__init__(self)
        self.loc = loc
        self.writing = False
        self.dirty = False

        sections = ['ui', 'osd', 'lastfm', 'equalizer',\
            'editor', 'import', 'replaygain', 'proxy', 'playlist']

        if os.path.exists(self.loc):
            try:
                self.read(self.loc)
            except ConfigParser.MissingSectionHeaderError:
                map(self.add_section, sections)

                from config_convert import ConvertIniToConf
                converter = ConvertIniToConf(self, self.loc)

        map(self.add_section, sections)

    def add_section(self, section_name="", plugin=None):
        section_name_str = section_name
        if plugin:
            section_name_str = "plugin/%s" % plugin

        try:
            SafeConfigParser.add_section(self, section_name_str)
        except ConfigParser.DuplicateSectionError:
            pass


    def get_section_key(self, key, plugin=None):
        plugin_str = plugin

        if plugin:
            if plugin.endswith(".py"):
                plugin_str = plugin[:-3]

        if plugin_str:
            if plugin_str not in self.sections():
                self.add_section(plugin=plugin_str)

            return ("plugin/%s" % plugin_str, key)
        else:
            split_str = key.split('/', 1)
            if len(split_str) > 1:
                return (split_str[0], split_str[1])
            else:
                return ('DEFAULT', split_str[0])

    def get_section_keyv(self, key, plugin, value):
        return self.get_section_key(key, plugin) + (value,)
    
    
    def get(self, key, default=None, plugin=None):
        """
            Gets a value from the settings.  If default is passed, and the value
            does not exist in the current settings, the default is returned
        """
        value = None

        try:
            value = SafeConfigParser.get(self, *self.get_section_key(key, plugin))
            return value
        except ConfigParser.NoOptionError:
            return default
    
    
    def get_str(self, key, default="", plugin=None):
        """
            Gets a string from the settings.  If default is passed, and the value
            does not exist in the current settings, the default is returned
        """
        value = self.get(key, plugin=plugin)

        if value is None:
            value = str(default)

        return value.replace(r'\n', '\n')
    
    
    def get_int(self, key, default=0, plugin=None):
        """
            Gets an int from the settings.  If default is passed, and the value
            does not exist in the current settings, the default is returned
        """
        value = self.get(key, plugin=plugin)

        if value is None:
            value = default
        else:
            try:
                value = int(value)
            except ValueError:
                value = default

        return value

    def get_float(self, key, default=0, plugin=None): 
        """
            Gets a float from the settings.  If default is passed, and the value
            does not exist in the current settings, the default is returned
        """
        value = self.get(key, plugin=plugin)

        if value is None:
            value = default
        else:
            try:
                value = float(value)
            except ValueError:
                value = default

        return value
    

    def get_boolean(self, key, default=False, plugin=None): 
        """
            Gets a bool from the settings.  If default is passed, and the value
            does not exist in the current settings, the default is returned
        """
        value = self.get(key, plugin=plugin)

        if value is None:
            value = default

        if value == "True" or value == "true":
            value = True
        elif value == "False" or value == "false":
            value = False
        else:
            value = default

        return value


    def get_list(self, key, default=None, plugin=None):
        """
            Gets a list from the settings.  If default is passed, and the value
            does not exist in the current settings, the default is returned
        """

        value = self.get(key, plugin=plugin)

        if value is None:
            value = default
        else:
            try:
                value = eval(value)
            except SyntaxError:
                value = default

        return value
    
    
    def set_boolean(self, key, value, plugin=None): 
        """
            Sets a boolean
        """
        if value == "true" or value == "True": value = True
        if value == "false" or value == "False": value = False
        
        self.set(*self.get_section_keyv(key, plugin, str(value)))
        self.dirty = True
    
    def set_str(self, key, value, plugin=None):
        """
            Sets a string
        """
        value_str = str(value)
        value_str = re.sub(r'%[^rdf%]?', r'%%', value_str)
        self.set(*self.get_section_keyv(key, plugin, value_str.replace('\n', r'\n')))
        self.dirty = True
    
    def set_int(self, key, value, plugin=None):
        """
            Sets an int
        """
        self.set(*self.get_section_keyv(key, plugin, str(value)))
        self.dirty = True
    
    def set_float(self, key, value, plugin=None):
        """
            Sets a float
        """
        self.set(*self.get_section_keyv(key, plugin, str(value)))
        self.dirty = True
    
    def set_list(self, key, value, plugin=None):
        """
            Sets a list
        """
        self.set(*self.get_section_keyv(key, plugin, str(value)))
        self.dirty = True
    
    def save(self): 
        """
            Saves the settings
        """
        if self.writing: return True
        if not self.dirty: return True
        self.writing = True
        self.dirty = False
        f = open(self.loc, 'w')
        self.write(f)
        f.close()
        self.writing = False
        return True
    
    def optionxform(self, option):
        return str(option)
    

class Config:
    """
        Allows for easy parsing of configuration files
    """
    def __init__(self, loc):
        """
            Expects the location of a file.  Opens the file, and parses it
            to populate the dictionary
        """
        self.config = XlConfigParser(loc)
        self.loc = loc

        # save settings every 30 seconds
        gobject.timeout_add(30000, self.save)

    def get_crypted(self, key, default="", plugin=None):
        """
            Gets a string from the config file and decrypts it
        """
        string = self.config.get_str(key, default, plugin)
        if not string: return default

        # convert it from hex
        new = ''
        vals = string.split()
        for val in vals:
            try:
                new += chr(int(val, 16))
            except ValueError:
                continue

        return self.crypt(new, XOR_KEY)

    def set_crypted(self, key, value, plugin=None):
        """
            Encrypts a value and saves it to the configuration file
        """
        writestr = self.crypt(value, XOR_KEY)

        hexstr = ''
        for x in writestr:
            hexstr = hexstr + "%02X " % ord(x)

        self.config.set_str(key, hexstr, plugin)

    def crypt(self, string, key):
        """
            Encrypt/Decrypt a string'
        """

        kidx = 0
        cryptstr = ""

        for x in range(len(string)):
            cryptstr = cryptstr + \
                chr(ord(string[x]) ^ ord(key[kidx]))

            kidx = (kidx + 1) % len(key)

        return cryptstr
    
    def get_str(self, key, default="", plugin=None):
        """
            Gets a string from the settings.  If default is passed, and the value
            does not exist in the current settings, the default is returned
        """
        return self.config.get_str(key, default, plugin)
    
    
    def get_int(self, key, default=0, plugin=None):
        """
            Gets an int from the settings.  If default is passed, and the value
            does not exist in the current settings, the default is returned
        """
        return self.config.get_int(key, default, plugin)

    def get_float(self, key, default=0.0, plugin=None): 
        """
            Gets a float from the settings.  If default is passed, and the value
            does not exist in the current settings, the default is returned
        """
        return self.config.get_float(key, default, plugin)
    

    def get_boolean(self, key, default=False, plugin=None): 
        """
            Gets a bool from the settings.  If default is passed, and the value
            does not exist in the current settings, the default is returned
        """
        return self.config.get_boolean(key, default, plugin)


    def get_list(self, key, default=[], plugin=None):
        """
            Gets a list from the settings.  If default is passed, and the value
            does not exist in the current settings, the default is returned
        """
        return self.config.get_list(key, default, plugin)
    
    
    def get_plugins(self):
        """
            Gets a dictionary of plugins with their status
        """
        plugins = {}

        plug_sections = []
        sections = self.config.sections()

        for section in sections:
            if 'plugin/' in section:
                plug_sections.append(section)

        for plug in plug_sections:
            plugin = plug.split('/')[1]
            plugins[plugin] = self.get_boolean("enabled", True, plugin=plugin)

        return plugins
    
    
    def set_boolean(self, key, value, plugin=None): 
        """
            Sets a boolean
        """
        self.config.set_boolean(key, value, plugin)
    
    
    def set_str(self, key, value, plugin=None):
        """
            Sets a string
        """
        self.config.set_str(key, value, plugin)
    
    
    def set_int(self, key, value, plugin=None):
        """
            Sets an int
        """
        self.config.set_int(key, value, plugin)
    
    
    def set_float(self, key, value, plugin=None):
        """
            Sets an int
        """
        self.config.set_float(key, value, plugin)
    
    
    def set_list(self, key, value, plugin=None):
        """
            Sets a list
        """
        self.config.set_list(key, value, plugin)
    
    def append_list(self, key, value, plugin=None):
        """
            Appends to a list
        """
        self.config.set_list(key, self.config.get_list(key, plugin=plugin) + [value])
    
    def save(self): 
        """
            Saves the settings
        """
        self.config.save()
        return True
    

    def __setitem__(self, key, value): 
        """
            sets a value and saves all values to the file
        """
        if isinstance(value, bool):
            self.set_boolean(key, value)
        elif isinstance(value, list):
            self.set_list(key, value)
        elif isinstance(value, float):
            self.set_float(key, value)
        elif isinstance(value, int):
            self.set_int(key, value)
        elif isinstance(value, str):
            self.set_str(key, value)

    def __getitem__(self, key):
        """
            gets a value
        """
        value = self.config.get(key)
        if value is None:
            return None
        elif _is_list(value):
            return self.get_list(key)
        elif _is_int(value):
            return self.get_int(key)
        elif _is_float(value):
            return self.get_float(key)
        elif _is_bool(value):
            return self.get_boolean(key)
        else:
            return self.get_str(key)

def _is_list(s):
    return s and s[0] == '[' and s[-1] == ']'
def _is_int(s):
    return s.isdigit()
def _is_float(s):
    try:
        float(s)
        return True
    except ValueError:
        return False
def _is_bool(s):
    return s in ('True', 'true', 'False', 'false')



#!/usr/bin/env python
# Copyright (C) 2006 Adam Olsen <arolsen@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

from xl import media, library, xlmisc, db, common
import os, xl, gobject, gtk, shutil, os.path
from gettext import gettext as _
import copy
import xl.plugins as plugins

PLUGIN_NAME = _("Mass Storage Driver")
PLUGIN_AUTHORS = ['Adam Olsen <arolsen@gmail.com>']
PLUGIN_VERSION = '0.3.4'
PLUGIN_DESCRIPTION = _(r"""Mass Storage Driver for the Devices Panel""")
PLUGIN_ENABLED = False
PLUGIN_ICON = None

PLUGIN = None

def configure():
    """
        Shows the configuration dialog
    """
    exaile = APP
    dialog = plugins.PluginConfigDialog(exaile.window, PLUGIN_NAME)
    table = gtk.Table(1, 2)
    table.set_row_spacings(2)
    bottom = 0
    label = gtk.Label(_("Mount Point:      "))
    label.set_alignment(0.0, 0.5)

    table.attach(label, 0, 1, bottom, bottom + 1)

    location = exaile.settings.get_str("mount", plugin=plugins.name(__file__),
        default="/mnt/device")

    loc_entry = gtk.FileChooserButton(_("Location"))
    loc_entry.set_action(gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER)
    loc_entry.set_current_folder(location)
    table.attach(loc_entry, 1, 2, bottom, bottom + 1, gtk.SHRINK)

    save_loc = exaile.settings.get_str('save_loc',
        plugin=plugins.name(__file__), default='')

    bottom += 1
    label = gtk.Label(_("Transfer Save Location:"))
    label.set_alignment(0.0, 0.5)
    table.attach(label, 0, 1, bottom, bottom + 1)

    save_entry = gtk.FileChooserButton(_("Save Directory"))
    save_entry.set_action(gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER)
    save_entry.set_current_folder(save_loc)
    table.attach(save_entry, 1, 2, bottom, bottom + 1, gtk.SHRINK)

    dialog.child.pack_start(table)
    dialog.show_all()

    result = dialog.run()
    dialog.hide()
    if result == gtk.RESPONSE_OK:
        exaile.settings.set_str("mount", loc_entry.get_current_folder(),
            plugin=plugins.name(__file__))
        exaile.settings.set_str('save_loc', loc_entry.get_current_folder(),
            plugin=plugins.name(__file__))

class MassStorageTrack(media.Track):
    type = 'device'
    pass

class MassStorageDriver(plugins.DeviceDriver):
    name = "massstorage"

    def __init__(self):
        plugins.DeviceDriver.__init__(self)
        self.db = None
        self.exaile = APP
        self.dp = APP.device_panel
        self.stopped = False

    def check_transfer_items(self, items):
        self.save_loc = self.exaile.settings.get_str('save_loc', default='', 
            plugin=plugins.name(__file__))
        if not self.save_loc:
            common.error(APP.window, _("You have not specified a directory "
                "to transfer your music in.  Please choose one in the "
                "configuration dialog"))
            configure()
            return False

        return True

    def remove_tracks(self, tracks):
        for track in tracks:
            try:
                self.all.remove(tracks)
                os.remove(track.io_loc)
            except:
                xlmisc.log_exception()

    def put_item(self, item):
        (item, nothing) = (item.track, item.target)
        if self.find_dup(item): return
        loc = os.path.basename(item.io_loc)

        try:
            shutil.copy(item.io_loc, os.path.join(self.save_loc, loc))
        except:
            xlmisc.log_exception()

        item = copy.deepcopy(item)
        item.loc = os.path.join(self.save_loc, loc)
        self.all.append(item)

    def find_dup(self, song):
        items = ('artist', 'album', 'genre', 'title', 'year')
        for s in self.all:
            check = False
            for item in items:
                if getattr(s, item) != getattr(song, item):
                    check = True

            if check: continue
            return True
        return False

    def remove_tracks(self, tracks):
        """
            Removes tracks
        """
        for track in tracks:
            if track in self.all: self.all.remove(track)
            try:
                os.unlink(track.io_loc)
            except OSError:
                pass
            except:
                xlmisc.log_exception()

    @common.threaded
    def connect(self, panel):
        """
            Connects and scans the device
        """
        self.stopped = False

        self.mount = self.exaile.settings.get_str("mount",
            plugin=plugins.name(__file__), default="/mnt/device")
        self.all = library.TrackData()

        files = library.scan_dir(str(self.mount), exts=media.SUPPORTED_MEDIA)
        for i, loc in enumerate(files):
            if self.stopped:
                self.stopped = False
                return
            tr = library.read_track(None, self.all, loc,
                track_type=MassStorageTrack)
            if not tr: continue
            self.all.append(tr)

        print 'we have connected, and scanned %d files!' % len(files)
        gobject.idle_add(panel.on_connect_complete, self)

    def search_tracks(self, keyword):

        if keyword:
            check = []
            for track in self.all:
                for item in ('artist', 'album', 'title'):
                    attr = getattr(track, item)
                    if keyword.lower() in attr.lower():
                        check.append(track) 
        else:
            check = self.all
        new = [(a.artist, a.album, a.track, a.title, a) for a in check]
        new.sort()
        return library.TrackData([a[4] for a in new])

    def disconnect(self):
        self.stopped = True

def initialize():
    global PLUGIN

    PLUGIN = MassStorageDriver()
    APP.device_panel.add_driver(PLUGIN, PLUGIN_NAME)

    return True

def destroy():
    global PLUGIN

    if PLUGIN:
        APP.device_panel.remove_driver(PLUGIN)

    PLUGIN = None



#!/usr/bin/env python

import re, os, cgi, time, urllib, sys
import os.path, shutil


name_re = re.compile(r'PLUGIN_NAME\s+=\s+r?(\'\'?\'?|""?"?)(.*?)(\1)', re.DOTALL|re.MULTILINE)
version_re = re.compile(r'PLUGIN_VERSION\s+=\s+r?(\'\'?\'?|""?"?)(.*?)(\1)', re.DOTALL|re.MULTILINE)
author_re = re.compile(r'PLUGIN_AUTHORS\s+=\s(\[.*?\])', re.DOTALL|re.MULTILINE)
description_re = re.compile(r'PLUGIN_DESCRIPTION\s+=\s+r?(\'\'?\'?|""?"?)(.*?)(\1)', re.DOTALL|re.MULTILINE)

out = open('plugin_info.txt', 'w+')

if not os.path.isdir('.htmain'):
    os.system('bzr checkout --lightweight http://bazaar.launchpad.net/~exaile-devel/exaile/main .htmain')
    os.chdir('.htmain/plugins')
else:
    os.chdir('.htmain/plugins')
    os.system('bzr update')


def get_plugin_info(file, data):
    m = name_re.search(data)
    if not m: return None

    name = m.group(2)

    m = version_re.search(data)
    if not m: return None

    version = m.group(2)

    m = author_re.search(data)
    if not m: return None

    author = eval(m.group(1))
    author = ", ".join(author)

    m = description_re.search(data)
    if not m: return None

    description = m.group(2)
    description = description.replace('\n', ' ')

    return (file, name, version, author, description)

files = os.listdir('.')
for file in files:
    if os.path.isdir(file):
        data = open('%s/%s.py' % (file, file)).read()
        inf = get_plugin_info('%s.exz' % file, data)

        os.chdir(file)
        os.system('zip -r ../../../%s.exz *' % file)
        os.chdir('..')
    else:
        data = open(file).read()
        shutil.copyfile(file, '../../%s' % file)
        inf = get_plugin_info(file, data)

    if not inf: continue
    out.write('\t'.join(inf) + '\n')

out.close()




import base64
import config
import socket
import string

True = 1
False = 0

class httpclient:

    def __init__(self, host, port = None):
        self.host = host
        self.port = port
        self.path = None
        self.status = None
        self.headers = None
        self.response = None
        self.debug = 0

        if self.host[:7] == "http://":
            self.host = self.host[7:]
            p = self.host.find("/")
            self.path = self.host[p:]
            self.host = self.host[:p]

        if not self.port:
            p = self.host.find(":")
            if p >= 0:
                self.port = int(self.host[p+1:])
                self.host = self.host[:p]
            else:
                self.port = 80

    def readline(self, s):
        res = ""
        while True:
            try:
                c = s.recv(1)
            except:
                break
            res = res + c
            if c == '\n':
                break
            if not c:
                break
        if self.debug:
            print "<<< " + res.replace("\n", "")
        return res

    def post(self, postdata):
        return self.req(None, postdata)

    def req(self, url=None, postdata=None):

        if not url:
            url = self.path

        if not url:
            print "httpclient: No URL?"
            return

        method = "GET"
        if postdata:
            method = "POST"

        s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        if config.useproxy:
            try:
                s.connect((config.proxyhost, config.proxyport))
            except:
                self.response = ""
                return

            req = method + " http://" + self.host + ":" + str(self.port) + url + " HTTP/1.1"
            if self.debug:
                print ">>> " + req
            s.send(req + "\r\n")
            if config.proxyuser != "":
                s.send("Proxy-Authorization: Basic " + base64.b64encode(config.proxyuser + ":" + config.proxypass) + "\r\n")
        else:
            try:
                s.connect((self.host, self.port))
            except:
                self.response = ""
                return
            req = method + " " + url + " HTTP/1.1"
            if self.debug:
                print ">>> " + req
            s.send(req + "\r\n")
        s.send("Host: " + self.host + "\r\n")
        s.send("Connection: close\r\n")

        if postdata:
            s.send("Content-Type: application/x-www-form-urlencoded\r\n")
            s.send("Content-Length: " + str(len(postdata)) + "\r\n")

        s.send("\r\n")

        if postdata:
            if self.debug:
                print ">>> postdata: " + postdata
            s.send(postdata)

        line = self.readline(s)
        self.status = string.rstrip(line)

        self.headers = {}
        while True:
            line = self.readline(s)
            if not line:
                break
            if line == "\r\n":
                break
            tmp = string.split(line, ": ")
            self.headers[tmp[0]] = string.rstrip(tmp[1])

        self.response = ""
        while True:
            line = self.readline(s)
            if not line:
                break
            self.response = self.response + line
        s.close()




#!/usr/bin/env python

# Copyright (C) 2006 Adam Olsen
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 1, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

import gtk, time, gobject, thread
from gettext import gettext as _
import xl.plugins as plugins

PLUGIN_NAME = _("Alarm Clock")
PLUGIN_AUTHORS = ['Adam Olsen <arolsen@gmail.com>', 'Christian Balles <me@ballessay.de>']
PLUGIN_VERSION = "0.2"
PLUGIN_DESCRIPTION = _(r"""Plays music at a specific time.\n\nNote that when the 
specified time arrives, Exaile will just act like you pressed the play button, 
so be sure you have the music you want to hear in your playlist""")

PLUGIN_ENABLED = False
PLUGIN = None
SETTINGS = None
TIMER_ID = None
RANG = dict()
APP = None

DAY_ID=0
DAY_NAME=1

days = [ ('1',   _('Monday') ),
         ('2',   _('Tuesday') ),
         ('3',   _('Wednesday') ),
         ('4',   _('Thursday') ),
         ('5',   _('Friday') ),
         ('6',   _('Saturday') ),
         ('0',   _('Sunday') ) ]

check_box_list = []

class VolumeControl:
    def __init__(self):
        self.thread = thread

    def print_debug( self ):
        print self.min_volume
        print self.max_volume
        print self.increment
        print self.time_per_inc

    def fade_in( self ):
        temp_volume = self.min_volume
        while temp_volume <= self.max_volume:
            #print "set volume to %s" % str(temp_volume / 100.0)
            APP.player.set_volume( ( temp_volume / 100.0 ) )
            temp_volume += self.increment
            time.sleep( self.time_per_inc )
            if APP.player.is_paused() or not APP.player.is_playing():
                self.stop_fading()
        

    def fade_out( self):
        temp_volume = self.max_volume
        while temp_volume >= self.min_volume:
            #print "set volume to %d" % (temp_volume / 100.0)
            APP.player.set_volume( ( temp_volume / 100.0) )
            temp_volume -= self.increment
            time.sleep( self.time_per_inc )
            if APP.player.is_paused() or not APP.player.is_playing():
                self.stop_fading()

    def fade_in_thread( self ):
        if self.use_fading == "True":
            self.thread.start_new( self.fade_in, ())

    def stop_fading( self ):
        self.thread.exit()

    def save_settings( self ):
        self.use_fading = str(self.use_button.get_active())
        self.min_volume = self.min_spinBox.get_value()
        self.max_volume = self.max_spinBox.get_value()
        self.increment  = self.inc_spinBox.get_value()
        self.time_per_inc = self.time_spinBox.get_value()
        APP.settings.set_str("alarm_use_fading", "%s" %  self.use_fading, plugin=plugins.name(__file__))
        APP.settings.set_str("alarm_min_volume", "%d" % self.min_volume, plugin=plugins.name(__file__))
        APP.settings.set_str("alarm_max_volume", "%d" % self.max_volume, plugin=plugins.name(__file__))
        APP.settings.set_str("alarm_increment", "%d" % self.increment, plugin=plugins.name(__file__))
        APP.settings.set_str("alarm_time_per_in", "%d" % self.time_per_inc, plugin=plugins.name(__file__))

    def load_settings( self ):
        self.use_fading     = APP.settings.get_str("alarm_use_fading", plugin=plugins.name(__file__), default="False")
        self.min_volume     = int(APP.settings.get_str("alarm_min_volume", plugin=plugins.name(__file__), default="0"))
        self.max_volume     = int(APP.settings.get_str("alarm_max_volume", plugin=plugins.name(__file__), default="100"))
        self.increment      = int(APP.settings.get_str("alarm_increment", plugin=plugins.name(__file__), default="1"))
        self.time_per_inc   = int(APP.settings.get_str("alarm_time_per_inc", plugin=plugins.name(__file__), default="1"))

    def create_vbox( self ):
        vbox = gtk.VBox()

        self.use_button = gtk.CheckButton( _( "Use Fading"))
        if self.use_fading == "True":
            self.use_button.set_active( True)
        vbox.pack_start( self.use_button, False, False)

        table = gtk.Table( 4, 2, True)

        table.attach( gtk.Label( _( "Minimum Volume:  ")), 0, 1, 0, 1)
        self.min_spinBox  = gtk.SpinButton( gtk.Adjustment( 1, step_incr=1))
        self.min_spinBox.set_range( 0, 100)
        self.min_spinBox.set_value( int(self.min_volume) )
        table.attach( self.min_spinBox, 1,2,0,1)

        table.attach( gtk.Label( _( "Maximum Volume:  ")), 0, 1, 1, 2)
        self.max_spinBox  = gtk.SpinButton( gtk.Adjustment( 1, step_incr=1))
        self.max_spinBox.set_range( 0, 100)
        self.max_spinBox.set_value( int(self.max_volume) )
        table.attach( self.max_spinBox, 1,2, 1,2)

        table.attach( gtk.Label( _( "Increment:  ")), 0, 1, 2, 3)
        self.inc_spinBox  = gtk.SpinButton( gtk.Adjustment( 1, step_incr=1))
        self.inc_spinBox.set_range( 1, 100)
        self.inc_spinBox.set_value( int(self.increment) )
        table.attach( self.inc_spinBox, 1, 2, 2, 3)

        table.attach( gtk.Label( _( "Time per Inc:  ")), 0, 1, 3, 4)
        self.time_spinBox = gtk.SpinButton( gtk.Adjustment( 1, step_incr=1))
        self.time_spinBox.set_range( 1, 100)
        self.time_spinBox.set_value( int(self.time_per_inc) )
        table.attach( self.time_spinBox, 1, 2, 3, 4)

        vbox.pack_start( table, False, False )

        return vbox

VOLUME_CONTROL = VolumeControl()

def create_alarm_time_hbox( hours = 12, minutes = 30):
    """
        Creates a GTK-HBox with spinboxes to set the alarm time
    """
    hbox = gtk.HBox()
    # TRANSLATORS: Alarm time for the Alarm Clock plugin
    hbox.pack_start(gtk.Label( _("Alarm Time:  ")), False, False)
    hour = gtk.SpinButton(gtk.Adjustment(1, step_incr=1))
    hour.set_range(0, 23)
    hour.set_value(int(hours))
    hbox.pack_start(hour, False, False)
    hbox.pack_start(gtk.Label(":"), False, False)
    minute = gtk.SpinButton(gtk.Adjustment(1, step_incr=1))
    minute.set_range(0, 59)
    minute.set_value(int(minutes))
    hbox.pack_start(minute, False, False)
    return (hbox, hour, minute)


def create_alarm_days_vbox( alarm_days ):
    """
        create a VBox with GTK-CheckBoxes to choose
        the days with alarm
    """
    del check_box_list[:]
    vbox = gtk.VButtonBox()

    for day in days:
        check_box_list.append( gtk.CheckButton( day[DAY_NAME]) )
        if is_day_active( day[DAY_ID], alarm_days) == True:
            check_box_list[-1].set_active(True)

        vbox.pack_start( check_box_list[-1] , False, False  )

    return vbox


def is_day_active( dayToSet, active_days_list):
    """
        is True if the dayToSet is in the activeDayList
    """
    for day in active_days_list:      
        if dayToSet == day:
            return True
    return False


def createDaySettingsString():
    """
        build a string with all the days the alarm is active
    """
    checked_days = ""
    i = 0
    for check_box in check_box_list:
        if check_box.get_active() is True:
            checked_days += (days[i])[0] + " "
        i += 1
    return checked_days
        

def configure():
    """
        Configures the time to ring
    """
    exaile = APP
    #get the alarm_time and alarm_days settings
    alarm_time = exaile.settings.get_str("alarm_time", plugin=plugins.name(__file__), default="12:30")
    alarm_days = exaile.settings.get_str("alarm_days", plugin=plugins.name(__file__), default="" )
    (hours, minutes) = alarm_time.split(":")
    alarm_day_list = []
    alarm_day_list = alarm_days.split(" ")

    VOLUME_CONTROL.load_settings()

    dialog = plugins.PluginConfigDialog(exaile.window, PLUGIN_NAME)

    notebook = gtk.Notebook()

    time_and_day_vbox = gtk.VBox()

    alarmTimeTupel = create_alarm_time_hbox( hours, minutes)
    time_and_day_vbox.pack_start( alarmTimeTupel[0], False, False)
    
    hour = alarmTimeTupel[1]
    minute = alarmTimeTupel[2]
    

    label = gtk.Label( _("Alarm Days:") )
    time_and_day_vbox.pack_start( label, False, False)

    alarmDaysBox = create_alarm_days_vbox( alarm_day_list)
    time_and_day_vbox.pack_start( alarmDaysBox, False, False)

    notebook.append_page( time_and_day_vbox, gtk.Label( _("Times")) )

    notebook.append_page( VOLUME_CONTROL.create_vbox(), gtk.Label( _( "Fading" )) )


    dialog.child.pack_start( notebook );

    dialog.show_all()
    result = dialog.run()
    dialog.hide()

    if result == gtk.RESPONSE_OK:
        hour = hour.get_value()
        minute = minute.get_value()
        exaile.settings.set_str("alarm_time", "%02d:%02d" % (hour, minute),
            plugin=plugins.name(__file__))
        exaile.settings.set_str("alarm_days", "%s" % createDaySettingsString(), plugin=plugins.name( __file__) )
        VOLUME_CONTROL.save_settings()

def timeout_cb():
    """
        Called every two seconds.  If the plugin is not enabled, it does
        nothing.  If the current time matches the time specified and the
        current day is selected, it starts playing
    """
    if not PLUGIN_ENABLED: return True

    alarm_time = SETTINGS.get_str("alarm_time", plugin=plugins.name(__file__), default="")
    alarm_days = SETTINGS.get_str("alarm_days", plugin=plugins.name(__file__), default="")

    if not alarm_time: return True
    if not alarm_days: return True

    current = time.strftime("%H:%M", time.localtime())
    currentDay = time.strftime("%w", time.localtime())

    if alarm_time == current and is_day_active( currentDay, alarm_days) == True:
        check = time.strftime("%m %d %Y %H:%M")
        if RANG.has_key(check): return True
        track = APP.player.current
        if track and (APP.player.is_playing() or APP.player.is_paused()): return True
        APP.player.play()
        VOLUME_CONTROL.fade_in_thread()

        RANG[check] = True

    return True

def initialize():
    """
        Starts the timer
    """
    global TIMER_ID, SETTINGS, APP
    exaile = APP
    SETTINGS = exaile.settings
    TIMER_ID = gobject.timeout_add(2000, timeout_cb)
    VOLUME_CONTROL.load_settings()
    return True

def destroy():
    """
        Stops the timer for this plugin
    """
    if TIMER_ID:
        gobject.source_remove(TIMER_ID)

icon_data = ["16 16 168 2",
"  	c None",
". 	c #666864",
"+ 	c #6E716C",
"@ 	c #6D706B",
"# 	c #6B6D69",
"$ 	c #646661",
"% 	c #6B6D68",
"& 	c #888B86",
"* 	c #838783",
"= 	c #747976",
"- 	c #6C716E",
"; 	c #6E716E",
"> 	c #737773",
", 	c #777A75",
"' 	c #626460",
") 	c #6D706A",
"! 	c #999D98",
"~ 	c #727774",
"{ 	c #919692",
"] 	c #B1B6B0",
"^ 	c #B2B7AF",
"/ 	c #B2B7B0",
"( 	c #9CA19C",
"_ 	c #686D6B",
": 	c #757974",
"< 	c #616561",
"[ 	c #696C66",
"} 	c #989B96",
"| 	c #797E7B",
"1 	c #B3B8B1",
"2 	c #DADDD6",
"3 	c #E5E8E3",
"4 	c #E9EBE7",
"5 	c #E7E9E5",
"6 	c #DFE2DD",
"7 	c #CACEC7",
"8 	c #717573",
"9 	c #737672",
"0 	c #575A58",
"a 	c #848782",
"b 	c #767A78",
"c 	c #B7BCB5",
"d 	c #E1E4DE",
"e 	c #EAECE8",
"f 	c #EBEDEA",
"g 	c #ECEEEB",
"h 	c #E6E8E5",
"i 	c #B0B2AE",
"j 	c #BCC0BA",
"k 	c #797C79",
"l 	c #6B6F6B",
"m 	c #444746",
"n 	c #5A5E5B",
"o 	c #7C807C",
"p 	c #9A9F9B",
"q 	c #E0E3DD",
"r 	c #EBEDE9",
"s 	c #EAECE9",
"t 	c #E5E6E3",
"u 	c #EEF0ED",
"v 	c #D5D6D4",
"w 	c #888887",
"x 	c #BCBCBB",
"y 	c #E7EAE5",
"z 	c #B3B7B2",
"A 	c #6A6D6A",
"B 	c #4A4D4C",
"C 	c #5E625F",
"D 	c #6E7270",
"E 	c #BEC2BD",
"F 	c #E8EAE6",
"G 	c #EDEFEB",
"H 	c #E4E6E3",
"I 	c #A1A2A0",
"J 	c #A5A5A4",
"K 	c #5F5F5E",
"L 	c #969795",
"M 	c #EAEBE9",
"N 	c #EFF1EE",
"O 	c #E0E2DF",
"P 	c #656966",
"Q 	c #535755",
"R 	c #666B68",
"S 	c #C4C7C3",
"T 	c #ECEEEA",
"U 	c #EFF0ED",
"V 	c #EFF0EE",
"W 	c #A1A1A0",
"X 	c #AFAFAF",
"Y 	c #616161",
"Z 	c #E4E5E3",
"` 	c #F4F5F3",
" .	c #F3F4F2",
"..	c #626663",
"+.	c #525654",
"@.	c #525756",
"#.	c #646866",
"$.	c #C8CAC7",
"%.	c #F1F2EF",
"&.	c #ECEDEB",
"*.	c #949594",
"=.	c #6D6D6D",
"-.	c #5A5A5A",
";.	c #D4D5D4",
">.	c #F7F8F6",
",.	c #F5F6F4",
"'.	c #646765",
").	c #4B504E",
"!.	c #474B4B",
"~.	c #B1B4B1",
"{.	c #F1F2F0",
"].	c #EBECEB",
"^.	c #CFD0CF",
"/.	c #E8E9E8",
"(.	c #C2C2C2",
"_.	c #B8B8B7",
":.	c #F2F2F1",
"<.	c #F6F7F5",
"[.	c #D2D4D2",
"}.	c #626662",
"|.	c #3F4443",
"1.	c #6F7371",
"2.	c #E7E9E7",
"3.	c #F2F3F1",
"4.	c #F5F6F5",
"5.	c #FAFAF9",
"6.	c #EDEEED",
"7.	c #F6F7F6",
"8.	c #939593",
"9.	c #585B58",
"0.	c #343939",
"a.	c #434847",
"b.	c #636763",
"c.	c #858886",
"d.	c #E2E3E2",
"e.	c #F7F8F7",
"f.	c #F9F9F9",
"g.	c #FBFBFB",
"h.	c #FCFCFC",
"i.	c #F4F5F4",
"j.	c #A2A4A3",
"k.	c #5C5F5B",
"l.	c #414544",
"m.	c #474C4B",
"n.	c #606461",
"o.	c #797C7B",
"p.	c #C6C7C7",
"q.	c #F3F4F3",
"r.	c #F7F7F7",
"s.	c #D8D9D8",
"t.	c #929492",
"u.	c #5A5D5A",
"v.	c #454948",
"w.	c #3D4242",
"x.	c #525553",
"y.	c #5C5F5C",
"z.	c #606361",
"A.	c #5B5F5C",
"B.	c #5C5F5D",
"C.	c #505350",
"D.	c #3D4140",
"E.	c #2F3437",
"F.	c #383C3D",
"G.	c #444847",
"H.	c #464A48",
"I.	c #424745",
"J.	c #393D3C",
"K.	c #313537",
"          . + @ # $             ",
"      % & * = - ; > , '         ",
"    ) ! ~ { ] ^ / ( _ : <       ",
"  [ } | 1 2 3 4 5 6 7 8 9 0     ",
"  a b c d e f g g h i j k l m   ",
"n o p q r s t u v w x y z A B   ",
"C D E F G H I J K L M N O P Q   ",
"n R S T U V W X Y Z `  .e ..+.  ",
"@.#.$.G %.&.*.=.-.;.>.,.g '.).  ",
"!.P ~.V {.].^./.(._.:.<.[.}.|.  ",
"  C 1.2.3.` 4.5.5.6.7.:.8.9.0.  ",
"  a.b.c.d.7.e.f.g.h.i.j.k.l.    ",
"    m.n.o.p.q.g.r.s.t.u.v.      ",
"      w.x.y.z.A.z.B.C.D.        ",
"        E.F.G.H.I.J.K.          ",
"                                "]
PLUGIN_ICON = gtk.gdk.pixbuf_new_from_xpm_data(icon_data)



import mutagen.flac
from xl import xlmisc

TYPE = 'flac'

VALID_TAGS = xlmisc.VALID_TAGS

def get_tag(flac, tag):
    try:    
        return [unicode(t) for t in flac[tag]]
    except KeyError:
        return [] 

def is_multi():
    return True

def fill_tag_from_path(tr):
    """
        Reads all tags from the file
    """
    try:
        f = mutagen.flac.FLAC(tr.io_loc)
    except:
        xlmisc.log("Couldn't read tags from file: " + tr.loc)
        return
    tr.length = int(f.info.length)

    for tag in VALID_TAGS:
        tr.tags[tag] = get_tag(f, tag)

def can_change(tag):
    """
        Can the tag in question be used in this file format?
    """
    return tag in VALID_TAGS

def write_tag(tr):
    f = mutagen.flac.FLAC(tr.io_loc)
    if f.vc is None: f.add_vorbiscomment()
    del(f.vc[:])

    for tag in VALID_TAGS:
        if tr.tags[tag]:
            f.vc[tag] = tr.tags[tag]

    #f.vc['artist'] = tr.artist
    #f.vc['album'] = tr.album
    #f.vc['title'] = tr.title
    #f.vc['discnumber'] = tr.disc_id
    #f.vc['genre'] = tr.genre
    #f.vc['tracknumber'] = str(tr.track)
    #f.vc['date'] = str(tr.date)
    #f.vc['version'] = tr.version
    #f.vc['performer'] = tr.performer
    #f.vc['copyright'] = tr.copyright
    #f.vc['organization'] = tr.org
    #f.vc['isrc'] = tr.isrc

    f.save()



# Copyright (C) 2006 Adam Olsen 
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 1, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

from gettext import gettext as _
import pygtk
pygtk.require('2.0')
import gtk, gtk.glade
import locale, os, time, threading, urllib, re

# python<2.5 compatibility. Drop this when python2.4 isn't used so much anymore.
try:
    any = any
except NameError:
    def any(seq):
        for e in seq:
            if e: return True
        return False

class MultiTextEntryDialog(gtk.Dialog):
    """
        Exactly like a TextEntryDialog, except it can contain multiple
        labels/fields.

        Instead of using GetValue, use GetValues.  It will return a list with
        the contents of the fields. Each field must be filled out or the dialog
        will not close.
    """
    def __init__(self, parent, title):
        gtk.Dialog.__init__(self, title, parent)


        self.hbox = gtk.HBox()
        self.vbox.pack_start(self.hbox, True, True)
        self.vbox.set_border_width(5)
        self.hbox.set_border_width(5)
        self.left = gtk.VBox()
        self.right = gtk.VBox()

        self.hbox.pack_start(self.left, True, True)
        self.hbox.pack_start(self.right, True, True)

        self.add_buttons(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
            gtk.STOCK_OK, gtk.RESPONSE_OK)

        self.fields = []

    def add_field(self, label):
        """
            Adds a field and corresponding label
        """
        label = gtk.Label(label + "     ")
        label.set_alignment(0, 0)
        label.set_padding(0, 5)
        self.left.pack_start(label, False, False)

        entry = gtk.Entry()
        entry.connect('activate', lambda *e:
            self.response(gtk.RESPONSE_OK))
        entry.set_width_chars(30)
        self.right.pack_start(entry, True, True)
        label.show()
        entry.show()

        self.fields.append(entry)

    def get_values(self):
        """
            Returns a list of the values from the added fields
        """
        return [unicode(a.get_text(), 'utf-8') for a in self.fields]

    def run(self):
        """
            Shows the dialog, runs, hides, and returns
        """
        self.show_all()
        response = gtk.Dialog.run(self)
        self.hide()
        return response

class TextEntryDialog(gtk.Dialog):
    """
        Shows a dialog with a single line of text
    """
    def __init__(self, parent, message, title):
        """
            Initializes the dialog
        """
        gtk.Dialog.__init__(self, title, parent)

        label = gtk.Label(message)
        label.set_alignment(0.0, 0.0)
        self.vbox.set_border_width(5)

        main = gtk.VBox()
        main.set_spacing(3)
        main.set_border_width(5)
        self.vbox.pack_start(main, True, True)

        main.pack_start(label, False, False)

        self.entry = gtk.Entry()
        self.entry.set_width_chars(35)
        main.pack_start(self.entry, False, False)

        self.add_buttons(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
            gtk.STOCK_OK, gtk.RESPONSE_OK)

        self.entry.connect('activate', 
            lambda e: self.response(gtk.RESPONSE_OK))

    def get_value(self):
        """
            Returns the text value
        """
        return unicode(self.entry.get_text(), 'utf-8')

    def set_value(self, value):
        """
            Sets the value of the text
        """
        self.entry.set_text(value)

    def run(self):
        self.show_all()
        response = gtk.Dialog.run(self)
        self.hide()
        return response

def to_unicode(x, default_encoding=None):
    if isinstance(x, unicode):
        return x
    elif default_encoding and isinstance(x, str):
        # This unicode constructor only accepts "string or buffer".
        return unicode(x, default_encoding)
    else:
        return unicode(x)

def tup(string, num):
    """
        returns a tuple with the first char of the string repeated 
        the number of times after the first char:
        ie: 5e would result in ('e', 'e', 'e', 'e', 'e')
    """
    a = []
    for i in range(num):
        a.append(string)
    return tuple(a)

def threaded(f):
    """
        A decorator that will make any function run in a new thread
    """
    def wrapper(*args, **kwargs):
        t = threading.Thread(target=f, args=args, kwargs=kwargs)
        t.setDaemon(True)
        t.start()

    wrapper.__name__ = f.__name__
    wrapper.__dict__ = f.__dict__
    wrapper.__doc__ = f.__doc__

    return wrapper

def synchronized(func):
    """
        A decorator to make a function synchronized - which means only one
        thread is allowed to access it at a time
    """
    def wrapper(self,*__args,**__kw):
        try:
            rlock = self._sync_lock
        except AttributeError:
            from threading import RLock
            rlock = self.__dict__.setdefault('_sync_lock',RLock())
        rlock.acquire()
        try:
            return func(self,*__args,**__kw)
        finally:
            rlock.release()
    wrapper.__name__ = func.__name__
    wrapper.__dict__ = func.__dict__
    wrapper.__doc__ = func.__doc__
    return wrapper

class idict(dict): 
    """
        Case insensitive dictionary
    """
    def __init__(self): 
        """
            Initializes the dictionary
        """
        dict.__init__(self)
        self.keys_dict = dict()
    

    def __setitem__(self, item, val): 
        """
            Sets an item in the dict
        """
        dict.__setitem__(self, item.lower(), val)
        self.keys_dict[item.lower()] = item
    

    def __getitem__(self, item): 
        """
            Gets an item from the dict
        """
        return dict.__getitem__(self, item.lower())
    

    def __contains__(self, key): 
        """
            Returns True if this dictionary contains the specified key
        """
        return self.has_key(key)
    

    def has_key(self, key): 
        """
            Returns True if this dictionary contains the specified key
        """
        return dict.has_key(self, key.lower())
    

    def keys(self): 
        """
            Returns the case sensitive values of the keys
        """
        return self.keys_dict.values()
    
class ldict(dict):
    """
        A dict that only handles lists
    """
    def __init__(self):
        dict.__init__(self)

    def __setitem__(self, item, value):
        if type(value) is not list: value = [value]
        dict.__setitem__(self, item , value)

    def __getitem__(self, item):
        try:
            return dict.__getitem__(self, item)
        except KeyError:
            return []

class ilist(list): 
    """
        Case insensitive list
    """
    def __init__(self): 
        """
            Initializes the list
        """
        list.__init__(self)
    

    def __contains__(self, item): 
        """
            Returns true if this list contains the specified item
        """
        for i in self:
            if i.lower() == item.lower():
                return True

        return False



# this code stolen from listen-gnome
""" Parse a date and return a time object """
""" Date like  Thu, 02 02 2005 10:25:21 ... """
def strdate_to_time(date):
    #removing timezone
    c = date[-5:-4]
    if (c == '+') or (c == '-'):
        date = date[:-6]

    #FIXME : don't remove use it in strptime
    c = date[-3:]
    if c in ["GMT","CST","EST","PST","EDT","PDT"]:
        date = date[:-3]

    #Remove day because some field have incorrect string
    c = date.rfind(",")
    if c!=-1:
        date = date [c+1:]
    date = date.strip()

    #trying multiple date formats
    new_date = None

    #Set locale to C to parse date
    locale.setlocale(locale.LC_TIME, "C")

    formats = ["%d %b %Y %H:%M:%S",#without day, short month
                "%d %B %Y %H:%M:%S",#without day, full month
                "%d %b %Y",#only date , short month
                "%d %B %Y",#only date , full month
                "%b %d %Y %H:%M:%S",#without day, short month
                "%B %d %Y %H:%M:%S",#without day, full month
                "%b %d %Y",#only date , short month
                "%B %d %Y",#only date , full month
                "%Y-%m-%d %H:%M:%S",
                ]
    for format in formats:
        try:
            new_date = time.strptime(date,format)
        except ValueError:
            continue

    locale.setlocale(locale.LC_TIME, '')
    if new_date is None:
        return ""

    return time.mktime(new_date)

def escape_xml(text):
    """
        Replaces &, <, and > with their entity references
    """
    # Note: the order is important.
    table = [('&', '&amp;'), ('<', '&lt;'), ('>', '&gt;')]
    for old, new in table:
        text = text.replace(old, new)
    return text

def to_url(path):
    """
        Converts filesystem path to URL. Returns the input unchanged if it's not
        an FS path (i.e. a URL or something invalid).
    """
    if re.search(r'^[\w]+://', path):
        return path
    try:
        return 'file://' + urllib.pathname2url(path)
    except IOError:
        return path

class ScrolledMessageDialog(gtk.Dialog):
    def __init__(self, parent, title):
        gtk.Dialog.__init__(self, title, parent)

        main = gtk.VBox()
        main.set_border_width(5)
        self.vbox.pack_start(main, True, True)

        self.add_buttons(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
            gtk.STOCK_OK, gtk.RESPONSE_OK)

        scroll = gtk.ScrolledWindow()
        scroll.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        scroll.set_shadow_type(gtk.SHADOW_IN)

        self.view = gtk.TextView()
        self.view.set_editable(False)
        scroll.add(self.view)

        main.pack_start(scroll, True, True)
        self.resize(500, 300)

    def run(self):
        self.show_all()
        response = gtk.Dialog.run(self)
        self.hide()
        return response

def scrolledMessageDialog(parent, message, title):
    """
        Shows a message dialog with a message in a TextView
    """
    dialog = ScrolledMessageDialog(parent, title)
    view = dialog.view
    view.get_buffer().set_text(message)

    buf = view.get_buffer()
    char = buf.get_char_count()
    iter = buf.get_iter_at_offset(char)
    view.scroll_to_iter(iter, 0)
    dialog.run()

def error(parent, message): 
    """
        Shows an error dialog
    """
    dialog = gtk.MessageDialog(parent, gtk.DIALOG_MODAL, gtk.MESSAGE_ERROR,
        gtk.BUTTONS_OK)
    dialog.set_markup(message)
    dialog.run()
    dialog.destroy()

def info(parent, message):
    """
        Shows an info dialog
    """
    dialog = gtk.MessageDialog(parent, gtk.DIALOG_MODAL, gtk.MESSAGE_INFO,
        gtk.BUTTONS_OK)
    dialog.set_markup(message)
    dialog.run()
    dialog.destroy()

def yes_no_dialog(parent, message):
    """
        Shows a question dialog and returns the result
    """
    dialog = gtk.MessageDialog(parent, 
        gtk.DIALOG_MODAL, gtk.MESSAGE_QUESTION, gtk.BUTTONS_YES_NO, 
        message)
    result = dialog.run()
    dialog.destroy()
    return result

class ShowOnceMessageDialog(gtk.Dialog):
    def __init__(self, title, parent, message, checked=True):
        gtk.Dialog.__init__(self, title, parent)

        vbox = gtk.VBox()
        vbox.set_border_width(5)
        vbox.set_spacing(3)
        self.vbox.pack_start(vbox, True, True)

        top = gtk.HBox()
        top.set_border_width(3)
        top.set_spacing(5)
        top.pack_start(gtk.image_new_from_stock(gtk.STOCK_DIALOG_WARNING,
            gtk.ICON_SIZE_DIALOG), False, False)
        label = gtk.Label()
        label.set_markup('<b>%s</b>' % message)
        label.set_alignment(0.0, 0.5)
        top.pack_start(label, True, True)

        vbox.pack_start(top, True, True)

        # TRANSLATORS: Checkbox for common dialogs
        self.box = gtk.CheckButton(_('Do not show this dialog again'))
        self.box.set_active(checked)
        vbox.pack_start(self.box)

        self.add_buttons(gtk.STOCK_OK, gtk.RESPONSE_OK)

    def run(self):
        self.show_all()
        result = gtk.Dialog.run(self)
        self.hide()
        return self.box.get_active()




# Copyright (C) 2006 Adam Olsen
# Copyright (C) 2007 Johannes Sasongko <sasongko@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

import sys, time, traceback

# Copied from common because common requires gtk.
def to_unicode(x, default_encoding=None):
    if isinstance(x, unicode):
        return x
    elif default_encoding and isinstance(x, str):
        # This unicode constructor only accepts "string or buffer".
        return unicode(x, default_encoding)
    else:
        return unicode(x)

def log(message, multi=False):
    """Log a single-line message."""
    try:
        message = to_unicode(message)
    except UnicodeDecodeError:
        log_stack("Can't decode: " + `message`)
        # Fall through.
    _log(message)

def log_multi(lines):
    """Log a multiline message."""
    for i, line in enumerate(lines):
        try:
            lines[i] = to_unicode(line)
        except UnicodeDecodeError:
            log_stack("Can't decode: " + `line`)
            # Fall through,
    _log(lines, multi=True)

def log_exception():
    """Log current exception."""
    message = file_and_line()
    message.extend(traceback.format_exc().split('\n'))
    _log(message, multi=True)

def log_stack(description=None, skip=1):
    """Log current stack."""
    message = file_and_line(skip)
    message.append('Stack:')
    stack = traceback.format_stack()
    for i in xrange(0, len(stack) - skip):
        message.append(stack[i].rstrip())
    if description:
        try:
            description = to_unicode(description)
        except UnicodeDecodeError:
            log_stack("Can't decode: " + `description`)
            # Fall through.
        message.append(description)
    _log(message, multi=True)

def file_and_line(skip=1):
    """Return a nicely formatted string containing current file and function."""
    co = sys._getframe(skip + 1).f_code
    return ["-----------------------",
            " %s ( %s @ %s):" % (co.co_name, co.co_filename, co.co_firstlineno),
            "-----------------------"]


# Light (console-only) logging.

queue = []

# Will be replaced by the heavy logger when it's initialized.
def _log(message, multi=False):
    timestamp = time.time()
    if multi:
        for line in message:
            console_log(line)
            queue.append((line, timestamp))
    else:
        console_log(message)
        queue.append((message, timestamp))

def console_log(message):
    try:
        print message
    except UnicodeEncodeError: # Can't encode to system encoding.
        print `message`


# Heavy logging.

gobject = None
gui = None

def init(exaile, filename):
    def inject_heavy_logging():
        global _log, gui, queue
        gui = LoggerGUI(exaile, filename)
        for message, timestamp in queue:
            gui.log(message, timestamp)
        _log = _heavy_log
        del queue
    global gobject
    import gobject
    gobject.idle_add(inject_heavy_logging)

def _heavy_log(message, multi=False):
    gobject.idle_add(_real_heavy_log, message, time.time(), multi)

def _real_heavy_log(message, timestamp, multi=False):
    if multi:
        for line in message:
            console_log(line)
            gui.log(line, timestamp)
    else:
        console_log(message)
        gui.log(message, timestamp)

class LoggerGUI:
    def __init__(self, exaile, filename):
        import gtk.glade
        self.exaile = exaile
        self.filename = filename
        xml = gtk.glade.XML('exaile.glade', 'DebugDialog', 'exaile')
        self.dialog = xml.get_widget('DebugDialog')
        self.dialog.set_transient_for(self.exaile.window)
        self.view = xml.get_widget('debug_textview')
        self.buf = buf = self.view.get_buffer()
        self.end_mark = buf.create_mark(None, buf.get_end_iter(), False)
        self.log_file = None
        xml.get_widget('debug_ok_button').connect('clicked', 
            lambda *e: self.dialog.hide())
        self.dialog.connect('delete_event', self._closed)

    def _closed(self, *e):
        self.dialog.hide()
        return True

    def __del__(self):
        if self.log_file:
            self.log_file.close()

    def log(self, message, timestamp=None):
        if not timestamp: timestamp = time.time()
        lt = time.localtime(timestamp)
        text = "[%s] %s\n" % (time.strftime("%H:%M:%S", lt), message)

        self.buf.insert(self.buf.get_end_iter(), text)

        if not self.log_file:
            try:
                self.log_file = open(self.filename, 'a')
            except:
                self.log_file = None

        if self.log_file:
            self.log_file.write(text)
            self.log_file.flush()

        self.view.scroll_to_mark(self.end_mark, 0)



import copy
from gettext import gettext as _
import gtk

import xlmisc, media, library, common

TAG_NO_ERROR, TAG_MISSING, TAG_DIFFERENT, TAG_UNSUPPORTED, TAG_MULTI = range(0, 5)

class TrackGroup(object):
    """
        Workhorse class for the tag editor... all operations are done
        to a group of tracks.

        When using this class, remember that everything is returned as
        a list, even if its length is only 1.
    """
    def __init__(self, tracks=None):
        """
            Classify the different properties of the group
        """
        self.tracks = []
        self.is_file = True # if all tracks are files
        self.multi = True   # if every track supports multiple tag values
        
        if not tracks: 
            return 

        for song in tracks:
            self.is_file &= song.is_file()
            self.multi &= song.is_multi()
        
        if not self.is_file:
            raise IsFileException('>0 tracks in the group are not files.')
            return

        self.tracks = tracks

    def __len__(self):
        return len(self.tracks)

    def __iter__(self):
        return iter(self.tracks)

    def pop(self, i):
        self.tracks.pop(i)

    def get_tag_status(self, tag):
        """
            Returns the status of a tag in this group. Values are either
            one of the errors listed above (TAG_MISSING etc) or a string
            if the tag is the same for all tracks
        """
        values = []
        change = True
        for song in self.tracks:
            change &= song.can_change(tag)
            v = song.tags[tag]
            if v: values.append(v)

        if not values: 
            return None

        if not change:
            # at least one track doesn't support this tag
            return TAG_UNSUPPORTED

        if len(values) < len(self.tracks):
            # some tracks are missing this tag
            return TAG_MISSING
        
        if not self.multi:
            # flag an error if we find multiple tag values without all tracks
            # supporting it
            if True in [len(v) > i for v in values]:
                return TAG_MULTI
            
        cmp = sorted(values[0])
        if False in [cmp == sorted(x) for x in values]:
            return TAG_DIFFERENT

        # if our poor tag has survived, it's the same for all tracks
        return values[0]

    def make_model(self):
        """
            Scan the tags from the files and construct a ListStore of them
        """
        TAGNAME, VALUE, OLDVALUE, ERROR, ADDED, REMOVED, EDITED = range(0, 7)
        model = gtk.ListStore(str, str, str, int, bool, bool, bool)

        for tag in xlmisc.VALID_TAGS:
            row = [tag, None, None, TAG_NO_ERROR, False, False, False]
            status = self.get_tag_status(tag)
            if status is None: 
                continue
            elif type(status) is int: 
                row[ERROR] = status
                model.append(row)
            elif type(status) is list:
                for v in status:
                    row[VALUE] = row[OLDVALUE] = v
                    model.append(row)

        return model
                
    def write_tags(self, exaile, model):
        """
            Write the changes to disk. Returns a list of strings (errors)
        """
        TAGNAME, VALUE, OLDVALUE, ERROR, ADDED, REMOVED, EDITED = range(0, 7)
        errors = []
        to_add = common.ldict()
        for track in self.tracks:
            track.rating, "woot woot"
            for row in model:
                tag = row[TAGNAME]
                if row[ERROR]:
                    if row[REMOVED]:
                        track.tags[tag] = []
                        continue
                else:
                    if row[REMOVED]:
                        if row[ADDED]: continue # add + remove = nothing
                        for i, v in enumerate(track.tags[tag]):
                            if v == row[VALUE]:
                                del(track.tags[tag][i])
                                break
                    elif row[ADDED]:
                        if to_add.has_key(tag) and not row[VALUE] in \
                            to_add[tag]:
                            to_add[tag].append(row[VALUE])
                        else:
                            to_add[tag] = row[VALUE]
                    elif row[EDITED]:
                        for i, v in enumerate(track.tags[tag]):
                            if v == row[OLDVALUE]:
                                track.tags[tag][i] = row[VALUE]

            for tag, val in to_add.iteritems():
                if track.tags.has_key(tag):
                    track.tags[tag].extend(val)
                else:
                    track.tags[tag] = val

            try:
                media.write_tag(track)
                library.save_track_to_db(exaile.db, track)

                try:
                    exaile.all_songs.remove(exaile.all_songs.for_path(track.loc))
                    exaile.all_songs.append(track)
                except:
                    xlmisc.log_exception()

                exaile.tracks.refresh_row(track)
            except:
                errors.append(_("Unknown error writing tag for %s") % track.loc)
                xlmisc.log_exception()

        exaile.tracks.queue_draw()
        exaile.db.db.commit()

        return errors



# Copyright (C) 2006 Adam Olsen
# Copyright (C) 2007 Sayamindu Dasgupta
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 1, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

from gettext import gettext as _

import ConfigParser, os
import gtk, gtk.glade, gst
import xl.path
from xl import xlmisc

def get_active_text(combobox):
    model = combobox.get_model()
    active = combobox.get_active()
    if active < 0:
        return None
    return unicode(model[active][0], 'utf-8')

class EqualizerWindow(object):
    """
        Equalizer Settings Window
    """
    def __init__(self, exaile):
        """
            Inizializes the window
        """
        self.exaile = exaile
        self.equalizer = exaile.player.equalizer
        self.xml = gtk.glade.XML('exaile.glade', 'EqualizerWindow', 'exaile')
        self.dialog = self.xml.get_widget('EqualizerWindow')
        self.dialog.set_transient_for(self.exaile.window)

        self.get_widgets()
        self.eq_enable_box.set_active(exaile.settings.get_boolean('equalizer/enabled',
            False))
        self.eq_enable_box.connect('toggled', self.toggle_enabled)

        self.delete.set_sensitive(False) # We can't let the user delete default presets
        self.save.set_sensitive(False)
        self.default_presets = []


        self._scale_changed_ids = []
        for scale in self.scales:
            self._scale_changed_ids.append(scale.connect('value_changed', self.on_value_changed_cb))
        self.close.connect('clicked', self.destroy)
        self.save.connect('clicked',  self.save_preset)
        self.delete.connect('clicked', self.delete_preset)
        self.import_btn.connect('clicked', self.import_preset)
        self._preset_changed_id = self.preset_chooser.connect('changed', self.preset_changed_cb)
        self.setup_preset_chooser()
        
        self.setup_equalizer()
        self.dialog.show()

    def destroy(self, *e):
        """
            Called when the close dialog is clicked
        """
        self.dialog.destroy()

    def toggle_enabled(self, widget):
        """
            Called when the user clicks the "Enable Equalizer" checkbox
        """

        # save it to the settings, and then reset the playbin
        self.exaile.settings.set_boolean('equalizer/enabled',
            self.eq_enable_box.get_active())

        xlmisc.log("Setting Equalizer support to %s" %
            self.eq_enable_box.get_active())
        player = self.exaile.player
        track = player.current

        position = 0
        if track is not None and player.is_playing():
            try:
                position = player.playbin.query_position(gst.FORMAT_TIME)[0]
            except gst.QueryError:
                position = 0
            player.stop(False)
            play_track = True
        else:
            play_track = False

        player.setup_playbin()
        player.audio_sink = None
        if play_track:
            player.play_track(track)
            player.current = track
            if position and track.type != 'stream':
                player.seek(position / gst.SECOND, False)

    def get_widgets(self):
        """
            Gets all widgets from the glade definition file
        """
        xml = self.xml
        self.scales = [xml.get_widget('eq_scale%02d' % i) for i in xrange(1, 11)]
        self.close = xml.get_widget('eq_close_button')
        self.preset_chooser = xml.get_widget('eq_preset_combobox')
        self.save = xml.get_widget('eq_save_button')
        self.delete = xml.get_widget('eq_del_button')
        self.import_btn = xml.get_widget('eq_import_button')
        self.eq_enable_box = xml.get_widget('eq_enable')

    def setup_equalizer(self):
        """
            Gets the previous equalizer values from the settings
        """
        self.band_values = self.exaile.settings.get_list('equalizer/band-values', 
            [0, 0, 0, 0, 0, 0, 0, 0, 0, 0])
        self.apply_band_values()
        self.update_scales()

    def update_scales(self):
        """
            Changes the values of the scales (updates the GUI)
        """
        i = 0
        # We need to block the callback for 'changed' first
        for id in self._scale_changed_ids:
            self.scales[i].handler_block(id)
            i = i + 1
        i = 0
        for scale in self.scales:
            scale.set_value(self.band_values[i])
            i = i + 1
        # Done. Now unblock the callbacks
        i = 0
        for id in self._scale_changed_ids:
            self.scales[i].handler_unblock(id)
            i = i + 1


    def on_value_changed_cb(self, *e):
        """
            Callback for value change in the scales
        """
        self.band_values = []
        for scale in self.scales: #FIXME: Not a very optimal way to do stuff
            self.band_values.append(scale.get_value())
        self.apply_band_values()
        if self.custom == True:
            self.preset_chooser.set_active(0)
        self.custom = True # Will be reset by apply_preset_by_name() if required 

    def preset_changed_cb(self, combobox):
         preset = get_active_text(combobox)
         self.apply_preset_by_name(preset)
         # The following block enables/disabled the save/delete button as and when reqd.
         if preset in self.default_presets:
             self.delete.set_sensitive(False)
             self.save.set_sensitive(False)
         else:
             self.delete.set_sensitive(True) # This is not a builtin preset, user can delete it
             self.save.set_sensitive(True)
         if preset == "Custom":
             self.delete.set_sensitive(False)

    def apply_preset_by_name(self, name):
        """ 
            Applies a given preset
        """
        if name == 'Custom':
            self.custom = True
        elif name in self.default_presets:
            self.custom = False
            config = ConfigParser.ConfigParser()
            config.read(['equalizer.ini'])
            presets = config.sections()
            presets.sort()
            for preset in presets:
                if config.get(preset, "name") == name:
                    self.band_values = eval(config.get(preset, "value"))
        else:
            option = 'equalizer/eqpreset_' + name.replace(' ', '_').lower()
            self.band_values = self.exaile.settings.get_list(option)[1:]           
                    
        self.exaile.settings.set_str('equalizer/last-preset', name)
        self.apply_band_values()
        self.update_scales()


    def setup_preset_chooser(self):
        config = ConfigParser.ConfigParser()
        config.read(['equalizer.ini']) # The default presets
        presets = config.sections()

        presets.sort()
        for preset in presets:
            self.preset_chooser.append_text(config.get(preset, "name"))
            self.default_presets.append(config.get(preset, "name"))

        cpresets = self.exaile.settings.config.options("equalizer") # The custom/imported presets
        cpresets.sort()
        for cpreset in cpresets:
            if 'eqpreset_' in cpreset:
                self.preset_chooser.append_text(self.exaile.settings.get_list(('equalizer/'+cpreset))[0])

        last_preset = self.exaile.settings.get_str('equalizer/last-preset')

        if last_preset == "Custom":
            self.custom = True
        else:
            self.custom = False

        # Block the signal handler apply_preset_by_name()
        self.preset_chooser.handler_block(self._preset_changed_id)

        model = self.preset_chooser.get_model()
        i = 0
        for m in model:
            if (m[0] == last_preset):
                self.preset_chooser.set_active(i)
            i = i + 1

        # Unblock the handler 
        self.preset_chooser.handler_unblock(self._preset_changed_id)

        # Enable/disable the buttons as and when required
        if last_preset in self.default_presets:
            self.delete.set_sensitive(False)
            self.save.set_sensitive(False)
        else:
            self.delete.set_sensitive(True) # This is not a builtin preset, user can delete it
            self.save.set_sensitive(True)
        if last_preset == "Custom":
            self.delete.set_sensitive(False)

        

    def apply_band_values(self):
        i = 0
        if self.equalizer != None:
            for v in self.band_values:
                self.equalizer.set_property(('band'+str(i)), v)
                i = i + 1
            self.exaile.settings.set_list('equalizer/band-values', self.band_values)
            return
        else:
            return

    def import_preset(self, *e):
        PresetImport(self)

    def save_preset(self, *e):
        PresetSave(self)

    def delete_preset(self, *e):

        #TODO: Add a confirmation Dialog

        name = get_active_text(self.preset_chooser)
        active = self.preset_chooser.get_active()

        # Delete the combobox entry first
        self.preset_chooser.remove_text(active)
        
        # Delete the settings from settings.ini
        option = 'eqpreset_' + name.replace(' ', '_').lower()
        self.exaile.settings.config.remove_option('equalizer', option)

        # Go back to Custom
        self.preset_chooser.set_active(0)
        self.apply_preset_by_name("Custom")
         
class PresetImport(object):
    def __init__(self, eq):
        """
            Imports a .eqf file
        """
        self.eq = eq
        self.xml = gtk.glade.XML('exaile.glade', 'EqualizerImportPreset', 'exaile')
        self.dlg = self.xml.get_widget('EqualizerImportPreset')
        self.dlg.set_transient_for(eq.dialog)

        self.name_widget = self.xml.get_widget('eq_preset_name_entry')

        self.import_btn = self.xml.get_widget('eq_preset_import_btn')
        self.import_btn.set_sensitive(False) # Since there's nothing in the text entry yet

        self.cancel_btn = self.xml.get_widget('eq_preset_cancel_btn')

        self.filechooser_btn = self.xml.get_widget('eq_preset_filechooser_btn')
        self.filechooser_btn.connect("selection-changed", self.enable_import)

        filter = gtk.FileFilter()
        filter.set_name(_("Winamp EQF files"))
        filter.add_pattern("*.eqf")
        self.filechooser_btn.add_filter(filter)
        filter = gtk.FileFilter()
        filter.set_name(_("All files"))
        filter.add_pattern("*")
        self.filechooser_btn.add_filter(filter)
        self.filechooser_btn.set_current_folder(xl.path.home)

        self.name_widget.connect('changed', self.enable_import)

        self.import_btn.connect('clicked', self.import_preset)
        self.cancel_btn.connect('clicked', lambda e: self.dlg.destroy())
        self.dlg.show_all()

    def enable_import(self, *e):
        name = unicode(self.name_widget.get_text(), 'utf-8')
        filename = self.filechooser_btn.get_filename()

        if (name and filename is not None):
            self.import_btn.set_sensitive(True)
        else:
            self.import_btn.set_sensitive(False)

    def import_preset(self, *e):
        filename = self.filechooser_btn.get_filename()

        band_values = self.parse_eqf(filename)

        for i in band_values:
            self.eq.band_values.append(i)

        name = self.name_widget.get_text()

        band_values.insert(0, name)
    
        name = name.replace(' ', '_').lower()
        #FIXME: Check for duplicate names
        self.eq.exaile.settings.set_list(('equalizer/eqpreset_'+name), band_values)

        self.eq.preset_chooser.append_text(band_values[0])
        self.eq.preset_chooser.set_active(len(self.eq.preset_chooser.get_model())-1)

        self.dlg.destroy()


    def parse_eqf(self, filename):
        # Stolen from XMMS

        f = open(filename, 'rb')
        header = f.read(31)

        if header.startswith('Winamp EQ library file v1.1'):
            tmp = f.read(257) # this contains the name - seems to be some problem with this
            b = f.read(11)

            preamp = 20.0 - ((ord(b[10])*40.0)/64.0)
    
            bands = []
            for i in b[0:10]:
                bands.append((20.0 - ((ord(i)*40.0)/64.0))/20.0)
        else:
            #TODO: Turn this into a proper error dialog
            print "The file does not seem to be a valid EQF file"

        return bands



        
class PresetSave(object):
    def __init__(self, eq):
        """
            Saves a preset in $HOME/.exaile/settings.ini
        """
        
        #TODO: Alert the user if the name is already being used.

        self.eq = eq

        self.xml = gtk.glade.XML('exaile.glade', 'EqualizerSavePreset', 'exaile')
        self.dlg = self.xml.get_widget('EqualizerSavePreset')
        self.dlg.set_transient_for(eq.dialog)

        self.name_widget = self.xml.get_widget('name_entry')
        self.save_btn = self.xml.get_widget('eq_save_save')
        self.save_btn.set_sensitive(False) # Since there's nothing in the text entry yet
        self.cancel_btn = self.xml.get_widget('eq_save_cancel')

        self.name_widget.connect('changed', self.enable_save)

        self.save_btn.connect('clicked', self.do_real_save_preset)
        self.cancel_btn.connect('clicked', lambda e: self.dlg.destroy())

        self.dlg.show_all()

    def do_real_save_preset(self, *e):
        band_values = []
        for scale in self.eq.scales: #FIXME: Not a very optimal way to do stuff
            band_values.append(scale.get_value())

        name = self.name_widget.get_text()

        band_values.insert(0, name)
    
        name = name.replace(' ', '_').lower()
        #FIXME: Check for duplicate names
        self.eq.exaile.settings.set_list(('equalizer/eqpreset_'+name), band_values)

        self.eq.preset_chooser.append_text(band_values[0])
        self.eq.preset_chooser.set_active(len(self.eq.preset_chooser.get_model())-1)

        self.dlg.destroy()

    def enable_save(self, *e):
        name = self.name_widget.get_text()
        self.save_btn.set_sensitive(bool(name))



# Copyright (C) 2006 Adam Olsen
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 1, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

__all__ = ['library', 'config', 'covers', 'db',
	'common', 'panels', 'xlmisc', 'prefs', 'track', 'media',
    'dbusinterface', 'player']



#!/usr/bin/env python

# Python console plugin for Exaile media player
# Copyright (C) 2007 Johannes Sasongko
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

import sys
import traceback
from cStringIO import StringIO
from gettext import gettext as _
import gtk

PLUGIN_NAME = _("Python Console")
PLUGIN_VERSION = '0.1.2'
PLUGIN_AUTHORS = ["Johannes Sasongko <sasongko@gmail.com>"]
PLUGIN_DESCRIPTION = _(r"""Provides a Python console that can be used to
manipulate Exaile.""")

PLUGIN_ICON = None
PLUGIN_ENABLED = False

class PyConsole(gtk.Window):
    def __init__(self, dict):
        gtk.Window.__init__(self)
        self.dict = dict

        self.buffer = StringIO()

        self.set_title(_("Python Console - Exaile"))
        self.set_border_width(12)
        self.set_default_size(450, 250)

        vbox = gtk.VBox(False, 12)
        self.add(vbox)

        sw = gtk.ScrolledWindow()
        vbox.pack_start(sw)
        sw.set_shadow_type(gtk.SHADOW_IN)
        self.text_view = tv = gtk.TextView()
        sw.add(tv)
        tv.set_editable(False)
        tv.set_wrap_mode = gtk.WRAP_CHAR
        self.text_buffer = buff = tv.get_buffer()
        self.end_mark = buff.create_mark(None, buff.get_end_iter(), False)

        hbox = gtk.HBox(False, 6)
        vbox.pack_start(hbox, False)
        label = gtk.Label('>>>')
        hbox.pack_start(label, False)
        self.entry = entry = gtk.Entry()
        hbox.pack_start(entry)
        entry.connect('activate', self.entry_activated)

        entry.grab_focus()
        vbox.show_all()

    def entry_activated(self, entry, user_data=None):
        """
            Called when the user presses Return on the GtkEntry.
        """
        self.execute(entry.get_text())
        entry.select_region(0, -1)

    def execute(self, code):
        """
            Executes some Python code.
        """
        stdout = sys.stdout
        try:
            pycode = compile(code, '<console>', 'single')
            sys.stdout = self.buffer
            exec pycode in self.dict
        except:
            sys.stdout = stdout
            exc = traceback.format_exception(*sys.exc_info())
            del exc[1] # Remove our function.
            result = ''.join(exc)
        else:
            sys.stdout = stdout
            result = self.buffer.getvalue()
            # Can't simply close and recreate later because help() stores and
            # reuses stdout.
            self.buffer.truncate(0) 
        result = '>>> %s\n%s' % (code, result)
        self.text_buffer.insert(self.text_buffer.get_end_iter(), result)
        # Can't use iter, won't scroll correctly.
        self.text_view.scroll_to_mark(self.end_mark, 0)

PLUGIN = None
MENU_ITEM = None

def initialize():
    global MENU_ITEM
    MENU_ITEM = gtk.MenuItem(_('Show Python Console'))
    MENU_ITEM.connect('activate', show_console)
    APP.xml.get_widget('tools_menu').get_submenu().append(MENU_ITEM)
    MENU_ITEM.show()
    return True

def destroy():
    global PLUGIN, MENU_ITEM
    if PLUGIN:
        PLUGIN.destroy()
        PLUGIN = None
    if MENU_ITEM:
        MENU_ITEM.hide()
        MENU_ITEM.destroy()
        MENU_ITEM = None

def show_console(*args):
    global PLUGIN
    if not PLUGIN:
        PLUGIN = PyConsole({'exaile': APP})
        PLUGIN.connect('destroy', console_destroyed)
    PLUGIN.present()

def console_destroyed(*args):
    global PLUGIN
    PLUGIN = None


if __name__ == '__main__':
    console = PyConsole({})
    console.connect('destroy', gtk.main_quit)
    console.show()
    gtk.main()

# vi: et ts=4 sts=4 sw=4



# Copyright (C) 2006 Adam Olsen
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 1, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

import os, re, traceback, sys, zipimport
from gettext import gettext as _
import xl.plugins, xl.xlmisc, xl.common

class Manager(object):
    """
        Plugin manager. 
        Manages loading plugins, and sending events to them
    """
    def __init__(self, app, update_cb=None):
        """
            Initializes the manager
        """
        self.plugins = []
        self.app = app
        self.loaded = []
        self.update_cb = update_cb

    def load_plugins(self, dir, enabled):
        """
            Loads all plugins in a specified directory
        """
        if not os.path.isdir(dir): return
        for file in os.listdir(dir):
            if file.endswith('.py') or file.endswith('.exz'):
                self.initialize_plugin(dir, file, enabled)

    def import_zip(self, dir, file):
        modname = file.replace('.exz', '')
        zip = zipimport.zipimporter(os.path.join(dir, file))
        try:
            plugin = zip.load_module(modname)
        except:
            xl.common.info(self.app.window, _('You may need to restart Exaile '
                'to completely update the %s plugin') % file)
            xl.xlmisc.log_exception()

        plugin.ZIP = zip
        plugin._IS_EXZ = True
        if hasattr(plugin, 'load_data'):
            plugin.load_data(zip)
            
        return plugin

    def initialize_plugin(self, dir, file, enabled=None, upgrading=False,
        avail_plugins=None):
        try:
            if file.endswith('.exz'):
                plugin = self.import_zip(dir, file)
                if not plugin: return
            else:
                oldpath = sys.path
                try:
                    sys.path.insert(0, dir)
                    plugin = __import__(re.sub('\.py[co]?$', '', file))
                finally:
                    sys.path = oldpath

            if not hasattr(plugin, "PLUGIN_NAME"):
                return

            if not hasattr(plugin, "PLUGIN_ENABLED"):
                plugin.PLUGIN_ENABLED = False

            if plugin.PLUGIN_NAME in self.loaded and upgrading: return
            if not upgrading:
                # check to see if this plugin is already installed and remove
                # it if that's the case
                for p in self.plugins:
                    if p.PLUGIN_NAME == plugin.PLUGIN_NAME:
                        p.destroy()
                        self.plugins.remove(p)

                if plugin.PLUGIN_NAME in self.loaded:
                    self.loaded.remove(plugin.PLUGIN_NAME)

            self.loaded.append(plugin.PLUGIN_NAME)
            
            print "Plugins '%s' version '%s' loaded successfully" % \
                (plugin.PLUGIN_NAME, plugin.PLUGIN_VERSION)

            plugin.FILE_NAME = file.replace('.exz', '.py')
            plugin.APP = self.app
           
            file = file.replace('.exz', '.py')

            if (enabled is None or file in enabled) or plugin.PLUGIN_ENABLED:
                if plugin.initialize():
                    plugin.PLUGIN_ENABLED = True
                    if self.update_cb: self.update_cb(plugin)

            # if this is a new plugin from a download (avail_plugins is only
            # set while downloading new plugins), check to see if it's
            # explicitly disabled in the settings.ini file.  If it's not
            # explicitly disabled, enable it.  We make the assumption that if
            # a user is downloading a plugin from the plugin manager, they
            # probably want it enabled by default.  Plugins manually put in
            # ~/.exaile/plugins will have to be enabled specifically
            elif (avail_plugins is not
                None and file not in avail_plugins):
                if plugin.initialize():
                    plugin.PLUGIN_ENABLED = True
                    if self.update_cb: self.update_cb(plugin)

            self.plugins.append(plugin)
        except xl.plugins.PluginInitException, e:
            self.plugins.append(plugin)
        except Exception, e:
            print "Failed to load plugin"
            traceback.print_exc()




# Port and address to listen to
listenport = 1881
bind_address = "127.0.0.1"

# Stick your last.fm username and password between the quotes below.
username = "synic"
password = "fruitloops"

# Which theme (skin) to use
theme = "default"

# Change "useproxy" to True and set the host and port if 
# you need an external proxy.
useproxy = False
proxyhost = "my.proxy.host"
proxyport = 8000
# Set these if your proxy requires authentication.
# Note: Only "Basic" authentication is supported.
proxyuser = ""
proxypass = ""




# Copyright (C) 2006 Adam Olsen 
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 1, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

import gtk, pango
from gettext import gettext as _
import common

class ConfigurationItem(object):
    """
        Represents a single configuration item
    """
    def __init__(self, ace, name, setting_type, value, description,
        change_func = None):
        """
            Sets up the configuration item
        """
        self.ace = ace
        self.name = name
        self.type = setting_type
        self._value = value
        self.description = description
        self.change_func = change_func

    def set_value(self, value):
        if self.type == 'float':
            try:
                self._value = float(value)
            except ValueError:
                self._value = 0.0
        elif self.type == 'int':
            try:
                self._value = int(value)
            except ValueError:
                self._value = 0
        elif self.type == 'list':
            try:
                self._value = eval(value)
            except:
                self._value = []
        elif self.type == 'boolean':
            if value and value.lower() == 'true':
                self._value = True
            else:
                self._value = False
        else:
            self._value = value

        if self.change_func:
            self.change_func(self, self._value)
        
        m = getattr(self.ace.settings, 'set_%s' % self.type.lower())
        m(self.name, self._value)
        print "setting %s to %s" % (self.name, self._value)
        if self.ace.eventhandler and hasattr(self.ace.eventhandler,
            'setting_changed'):
            self.ace.eventhandler.setting_changed()

    def get_value(self):
        return self._value

    value = property(get_value, set_value)

class AdvancedConfigEditor(gtk.Window):
    """
        Show an advanced configuration dialog
    """
    def __init__(self, app, parent, eventhandler, settings, metafile):
        gtk.Window.__init__(self)
        self.set_title(_('Advanced Configuration Editor'))
        self.set_transient_for(parent)
        self.app = app
        self.eventhandler = eventhandler
        self.settings = settings
        self.metafile = metafile
        self.items = []

        h = open(metafile)
        lines = h.readlines()
        h.close()

        for line in lines:
            (name, type, default, description, changefunc) = line.strip().split('\t')
            m = getattr(settings, 'get_%s' % type.lower())
            current = m(name, default)

            handler = None
            if changefunc and changefunc != 'none' and hasattr(eventhandler, 'advanced_%s' % 
                changefunc):
                handler = getattr(eventhandler, 'advanced_%s' % changefunc)

            self.items.append(ConfigurationItem(self, name, type, current,
                description, handler))

        main = gtk.VBox()
        main.set_border_width(3)
        main.set_spacing(3)
        self.add(main)

        top_box = gtk.HBox()
        top_box.pack_start(gtk.Label(_('Filter:') + '  '), False, False)
        self.filter = gtk.Entry()
        self.filter.connect('activate', lambda *e: self.populate_model())
        top_box.pack_start(self.filter, True, True)

        main.pack_start(top_box, False, False)

        self.model = gtk.ListStore(object, str, str, str)
        self.populate_model()

        scroll = gtk.ScrolledWindow()
        scroll.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        scroll.set_shadow_type(gtk.SHADOW_IN)

        self.tree = gtk.TreeView(self.model)
        self.tree.connect('row-activated', self.row_activated)
        self.tree.get_selection().connect('changed',
            self.on_selection_changed)

        # set up the treeview
        text = gtk.CellRendererText()
        # TRANSLATORS: Name of a setting in the Advanced Configuration Editor
        col = gtk.TreeViewColumn(_('Setting Name'))
        col.pack_start(text, False)
        col.set_sizing(gtk.TREE_VIEW_COLUMN_AUTOSIZE)

        col.set_attributes(text, text=1)
        self.tree.append_column(col)

        self.value_text = gtk.CellRendererText()
        self.value_text.set_property('editable', True)
        self.value_text.connect('edited', self.edited_cb)
        col = gtk.TreeViewColumn(_('Value'))
        col.set_expand(True)
        col.set_sizing(gtk.TREE_VIEW_COLUMN_FIXED)
        col.set_fixed_width(1)
        self.value_text.set_property('ellipsize', pango.ELLIPSIZE_END)
        col.pack_start(self.value_text, True)
        col.set_attributes(self.value_text, text=2)
        col.set_cell_data_func(self.value_text, self.value_data_func)
        self.tree.append_column(col)

        text = gtk.CellRendererText()
        col = gtk.TreeViewColumn(_('Type'))
        col.set_sizing(gtk.TREE_VIEW_COLUMN_AUTOSIZE)
        col.pack_start(text, True)
        col.set_attributes(text, text=3)
        self.tree.append_column(col)

        scroll.add(self.tree)

        main.pack_start(scroll, True, True)
        label = gtk.Label()
        label.set_markup('<b>' + _('Description:') + '</b>')
        label.set_alignment(0, 0)
        main.pack_start(label, False)

        self.desc_label = gtk.Label()
        self.desc_label.set_alignment(0, 0)
        main.pack_start(self.desc_label, False)

        buttons = gtk.HBox()
        close = gtk.Button(_('Close'), gtk.STOCK_CLOSE)
        close.connect('clicked', lambda *e: self.destroy())
        buttons.pack_end(close, False, False)

        main.pack_start(buttons, False, False)

        self.resize(635, 500)
        self.set_position(gtk.WIN_POS_CENTER_ON_PARENT)
        self.show_all()

    def on_selection_changed(self, selection):
        """
            Called when an item in the list is selected (so that we can
            populate the description label
        """
        if selection.get_selected():
            (model, iter) = selection.get_selected()
            if not iter:
                self.desc_label.set_label('')
                return
            item = model.get_value(iter, 0)
            self.desc_label.set_label(item.description)
        else:
            self.desc_label.set_label('')

    def value_data_func(self, column, cell, model, iter):
        """
            Value data func
        """
        type = model.get_value(iter, 3)
        if type == 'Boolean':
            cell.set_property('editable', False)
        else:
            cell.set_property('editable', True)
        cell.set_property('text', model.get_value(iter, 2))

    def row_activated(self, tree, path, col):
        """
            Called when the user clicks on the TreeView
        """
        iter = self.model.get_iter(path)
        item = self.model.get_value(iter, 0)

        if item.type.lower() == 'boolean':
            item.value = not item.value
            self.model.set_value(iter, 2, item.value)
        else: return True

    def validate(self, type, value):
        """
            Validates a value for a given type
        """
        if type == 'Int':
            value = int(value)
        elif type == 'Float':
            value = float(value)
        elif type == 'List':
            value = eval(value)

    def edited_cb(self, cell, path, new_text, data=None):
        """
            Called when one of the cells is edited
        """
        iter = self.model.get_iter(path)
        item = self.model.get_value(iter, 0)

        try:
            self.validate(item.type, new_text)
        except:
            common.error(self.parent, _('Invalid setting for %s' %
                item.name))
            return

        item.value = new_text
        self.model.set_value(iter, 2, new_text)

    def populate_model(self):
        """
            Populates the model with the settings items
        """
        self.model.clear()
        filter = unicode(self.filter.get_text(), 'utf-8')
        for item in self.items:
            if filter:
                if item.name.lower().find(filter.lower()) == -1: continue
            self.model.append([item, item.name, item.value, item.type])



# Copyright (C) 2006 Adam Olsen 
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 1, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

import threading, re, os, traceback, xlmisc
import tempfile
import shutil
try:
    from sqlite3 import dbapi2 as sqlite
    SQLITE_AVAIL = True
except ImportError:
    try:
        from pysqlite2 import dbapi2 as sqlite
        SQLITE_AVAIL = True
    except ImportError:
        SQLITE_AVAIL = False

#sqlite.enable_shared_cache(True)

import gobject

class DBOperationalError(Exception):
    
    
    def __init__(self, message):
        """ Create a new DBOperationalError

            :Parameters:
                - `message`: The message that will be displayed to the user
        """
        self.message = message
    
    def __repr__(self):
        msg = "%s: %s"
        return msg % (self.__class__.__name__, self.message,)
    
    def __str__(self):
        return self.__repr__()

class DBManager(object):
    """
        Manages the database connection
    """
    def __init__(self, db_loc, start_timer=True):
        """
            Initializes and connects to the database
        """
        
        self.db_loc = db_loc
        self.db = self.__get_db()

        self.pool = dict()
        self.timer_id = None
        self.p = '?'
        self.functions = []
        if start_timer:
            self.timer_id = gobject.timeout_add(2 * 60 * 60, self.commit)

        cur = self.db.cursor()
        cur.execute("PRAGMA synchronize=OFF")
        cur.execute("PRAGMA count_changes=0")
        cur.execute("PRAGMA auto_vacuum=1")
        cur.execute("PRAGMA cache_size=4000")
        cur.execute("PRAGMA temp_store=MEMORY")
        cur.execute("PRAGMA fullsync=0")
        cur.execute("PRAGMA case_sensitive_like=0")
        cur.close()
        self._cursor = self.db.cursor()

    def add_function_create(self, tup):
        """
            Adds a function that will be created in all pooled dbs
        """
        self.db.create_function(tup[0], tup[1], tup[2])
        self.functions.append(tup)

    def cursor(self, new=False):   
        """
            Returns the write cursor
        """
        if new:
            return self._get_from_pool().cursor()
        else:
            return self._cursor
        
    def __get_db(self):
        """
            Returns a connection
        """
        try:
            db = sqlite.connect(self.db_loc, check_same_thread=False)
            db.text_factory = lambda x: unicode(x, 'utf-8', 'ignore')
        except sqlite.OperationalError, e:
            raise DBOperationalError(str(e))

        return db

    def close(self):
        """
            Closes the connection
        """
        if self.timer_id:
            gobject.source_remove(self.timer_id)

    def check_version(self, path, echo=True):
        """
            Checks the database version and updates it if necessary
        """
        version = 0
        row = self.read_one("db_version", "version", "1=1", tuple())
        if row:
            version = int(row[0])

        files = os.listdir(path)
        files.sort()

        versions = []
        for file in files:
            m = re.search("changes(\d+)\.sql", file)
            if not m: continue
            ver = int(m.group(1))
            file = "changes%.4d.sql" % ver
            if ver > version:
                if echo:
                    xlmisc.log("Importing sql changes file %s" % file)
                self.import_sql("%s/%s" % (path, file))

    def import_sql(self, file):
        """
            Imports an SQL file
        """
        lines = []
        for line in open(file):
            if not line.startswith("--"):
                lines.append(line)

        data = "".join(lines)
        data = data.replace("\n", " ")
        lines = data.split(";")

        for line in lines:
            if line.strip():
                self.execute(line)
        self.db.commit()

    def execute(self, query, args=None):
        """
            Executes a query on the main event loop
        """
        if threading.currentThread().getName() == 'MainThread':
            self._execute(query, args)
        else:
            gobject.idle_add(self._execute, query, args)
        
    def _execute(self, query, args=None):
        """
            Executes a query
        """
        cur = self._cursor
        if not args: args = []
        try:
            cur.execute(query, args)
        except:
            traceback.print_exc()
            traceback.print_stack()
            print query, args

    def select(self, query, args=[]):
        """
            Runs a select and returns all rows.  This is only for small
            select operations.  If you want to do a large select, use
            DBManager.realcursor()
        """
        db = self._get_from_pool()
        cur = db.cursor()
        cur.execute(query, args)
        rows = []

        while True:
            try:
                row = cur.fetchone()
                if not row: break
                rows.append(row)
            except:
                xlmisc.log_exception()

        cur.close()

        return rows

    def realcursor(self):
        """
            Returns a new cursor from the database
        """
        return self.db.cursor()

    def record_count(self, table, where, args): 
        """
            Returns the number of rows matched for the query in the database
        """
        db = self._get_from_pool()
        cur = db.cursor()
        cur.execute("SELECT count(*) AS c FROM %s WHERE %s" % (table, where), args)
        row = cur.fetchone()
        cur.close()
        return row[0]

    def commit(self):
        """
            Commits the database
        """
        if threading.currentThread().getName() == 'MainThread':
            self.db.commit()
        else:
            gobject.idle_add(self.db.commit)

    def _get_from_pool(self):
        """
            Returns a database connection specific to the current thread
        """
        name = threading.currentThread().getName()
        if name == "MainThread" or self.db_loc == ":memory:": return self.db
        if not self.pool.has_key(name):
            db = self.__get_db()
            for tup in self.functions:
                db.create_function(tup[0], tup[1], tup[2])
            self.pool[name] = db
            print "Created db for thread %s" % name
            print self.pool

        db = self.pool[name]
        return db

    def _close_thread(self):
        """
            Closes the db in the pool for the current thread
        """
        name = threading.currentThread().getName()
        if name == "MainThread": return
        if self.pool.has_key(name):
            self.pool[name].close()
            del self.pool[name]
            print "Closed db for thread %s" % name

    def read_one(self, table, items, where, args):
        """
            Returns the first row matched for the query in the database
        """
        cur = self.db.cursor()
        query = "SELECT %s FROM %s WHERE %s LIMIT 1" % \
           (items, table, where)

        cur.execute(query, args)   
        row = cur.fetchone()

        cur.close()
        return row

    def update(self, table, vals, where, args, new=False):
        """
            Updates a table or creates a new row
        """
        if threading.currentThread().getName() == 'MainThread':
            self._update(table, vals, where, args, new)
        else:
            gobject.idle_add(self._update, table, vals, where, args, new)

    def _update(self, table, vals, where, args, new=False): 
        """
            Updates the database based on the query... or, if the specified row
            does not currently exist in the database, a new row is created
        """
        cur = self._cursor

        if new:
            keys = vals.keys()
            values = []
            for key in keys:
                val = vals[key]
                if not isinstance(val, basestring):
                    val = unicode(val)

                val = val.decode("utf-8", 'replace')
                values.append(val)

            left = ", ".join(keys)
            right = ", ".join(['?' for x in keys])
            query = "REPLACE INTO %s( %s ) VALUES( %s )" % (table, left, right)
            try:
                cur.execute(query, tuple(values))
            except Exception, e:
                print "Error on: %s" % query
                print e
            return
        else:
            values = []
            keys = []
            for k, v in vals.iteritems():

                if not isinstance(v, str): v = str(v)
                v = v.decode("utf-8", 'replace')
                keys.append("%s=?" % k)
                values.append(v)

            values.extend(args)

            query = "UPDATE %s SET %s WHERE %s" % (table, ", ".join(keys), where)
            cur.execute(query, values)

def insert_id(cur):
    cur.execute('SELECT LAST_INSERT_ROWID()')
    row = cur.fetchone()
    return row[0]

## Exaile specific code, for converting a 0.2.6 database to a 0.2.7
## hopefully we can remove it soon...
def get_column_id(cur, table, col, value):
    cur.execute('SELECT id FROM %s WHERE %s=?' % (table, col), (value,))
    row = cur.fetchone()
    if not row:
        cur.execute('INSERT INTO %s(%s) VALUES(?)' % (table, col), (value,))
        return insert_id(cur) 
    else: return row[0]

def get_album_id(cur, artist_id, album):
    cur.execute('SELECT id FROM albums WHERE artist=? AND name=?', (artist_id,
        album))

    row = cur.fetchone()
    if not row:
        cur.execute('INSERT INTO albums(artist, name) VALUES(?, ?)',
            (artist_id, album))
        return insert_id(cur)
    else: return row[0]
    
def convert_to027(loc):
    (h, name) = tempfile.mkstemp()
    shutil.move(loc, name)

    print name, loc
    old = DBManager(name)
    db = DBManager(loc)
    db.import_sql('sql/db.sql')

    oldcur = old.realcursor()
    new = db.realcursor()

    # convert tracks
    oldcur.execute('SELECT path, artist, album, title, genre, year, track, '
        'length, bitrate, modified, tags, plays, rating, user_rating, '
        'blacklisted FROM tracks')
    
    # tracks
    for row in oldcur.fetchall():
        row = list(row)
        row[0] = get_column_id(new, 'paths', 'name', row[0])
        row[1] = get_column_id(new, 'artists', 'name', row[1])
        row[2] = get_album_id(new, row[1], row[2])

        new.execute('INSERT INTO tracks(path, artist, album, title, genre, year, track, '
            'length, bitrate, modified, tags, plays, rating, user_rating, '
            'blacklisted) VALUES(%s)' % ','.join(['?' for x in row]), row)

    # album images
    oldcur.execute('SELECT image, artist, album FROM albums WHERE image IS NOT'
        ' NULL AND image!="" AND image NOT LIKE "%%nocover%%"')

    for row in oldcur.fetchall():
        artist_id = get_column_id(new, 'artists', 'name', row[1])
        album_id = get_album_id(new, artist_id, row[2])
        new.execute('UPDATE albums SET image=? WHERE artist=? AND id=?',
            (row[0], artist_id, album_id))

    # playlists
    oldcur.execute('SELECT playlist_name FROM playlists')
    playlists = oldcur.fetchall()
    oldcur.execute('SELECT playlist, path FROM playlist_items')
    playlist_items = oldcur.fetchall()
 
    for playlist in playlists:
        playlist_id = get_column_id(new, 'playlists', 'name', playlist[0])
        for item in playlist_items:
            if item[0] == playlist[0]:
                path_id = get_column_id(new, 'paths', 'name', item[1])
                new.execute(
                    "INSERT INTO playlist_items(playlist, path) VALUES(?, ?)",
                    (playlist_id, path_id))

    new.close()
    db.db.commit()
    return db



# Copyright (C) 2006-2007 Aren Olson
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 1, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

import os, gtk, dbus
import xl.plugins as plugins
from xl import panels, media, xlmisc, library, common
from gettext import gettext as _
from xl.panels import collection
import xl.path

try:
    import daap
    from daap import DAAPClient
    DAAP = True
except:
    DAAP = False

#detect authoriztion support in python-daap
if DAAP:
    try:
        tmp = DAAPClient()
        tmp.connect("spam","eggs","sausage") #dummy login
        del tmp
    except TypeError:
        AUTH = False
    except:
        AUTH = True

try:
    import avahi
    AVAHI = True
except:
    AVAHI = False

PLUGIN_NAME = _("Music Sharing")
PLUGIN_AUTHORS = ['Aren Olson <reacocard@gmail.com>']
PLUGIN_VERSION = '0.8.2'
PLUGIN_DESCRIPTION = _(r"""Allows playing of DAAP music shares.
\n\nDepends: python-daap, python-avahi.""")

PLUGIN_ENABLED = False
PLUGIN_ICON = gtk.Button().render_icon('gtk-network', gtk.ICON_SIZE_MENU)

APP = None
SETTINGS = None
TAB_PANE = None
PANEL = None
AVAHI_INTERFACE = None
CONNECTIONS = {}

# Have to use a string since we only have one file.
GLADE_XML_STRING = None

def load_data(zip):
    """
        Called by Exaile to load the data from the zipfile
    """
    global GLADE_XML_STRING

    GLADE_XML_STRING = zip.get_data('gui.glade')

class DaapAvahiInterface: #derived from python-daap/examples
    """
        Handles detection of DAAP shares via Avahi.
    """

    def new_service(self, interface, protocol, name, type, domain, flags):
        interface, protocol, name, type, domain, host, aprotocol, address, port, txt, flags = self.server.ResolveService(interface, protocol, name, type, domain, avahi.PROTO_UNSPEC, dbus.UInt32(0))
        """
            Called when a new share is found.
        """
#        print "DAAP: Found %s." % name
        #Use all available info in key to avoid name conflicts.
        nstr = '%s%s%s%s%s' % (interface, protocol, name, type, domain)
        CONNECTIONS[nstr] = DaapConnection(name, address, port)
        self.panel.update_connections()

    def remove_service(self, interface, protocol, name, type, domain, flags):
        """
            Called when the connection to a share is lost.
        """
#        print "DAAP: Lost %s." % name
        nstr = '%s%s%s%s%s' % (interface, protocol, name, type, domain)
        del CONNECTIONS[nstr]
        self.panel.update_connections()

    def __init__(self, panel):
        """
            Sets up the avahi listener.
        """
        self.bus = dbus.SystemBus()
        self.server = dbus.Interface(self.bus.get_object(avahi.DBUS_NAME,
            avahi.DBUS_PATH_SERVER), avahi.DBUS_INTERFACE_SERVER)
        self.stype = '_daap._tcp'
        self.domain = 'local'
        self.browser = dbus.Interface(self.bus.get_object(avahi.DBUS_NAME,
            self.server.ServiceBrowserNew(avahi.IF_UNSPEC, avahi.PROTO_UNSPEC,
            self.stype, self.domain, dbus.UInt32(0))),
            avahi.DBUS_INTERFACE_SERVICE_BROWSER)
        self.browser.connect_to_signal('ItemNew', self.new_service)
        self.browser.connect_to_signal('ItemRemove', self.remove_service)
        self.panel = panel

class DaapConnection(object):
    """
        A connection to a DAAP share.
    """
    def __init__(self, name, server, port):
        self.all = library.TrackData()
        self.connected = False
        self.tracks = None
        self.server = server.encode('ascii')
        self.port = port.encode('ascii')
        self.name = name
        self.auth = False
        self.password = None

    def connect(self, password = None):
        """
            Connect, login, and retrieve the track list.
        """
        try:
            client = DAAPClient()
            if AUTH and password:
                client.connect(self.server, self.port, password)
            else:
                client.connect(self.server, self.port)
            self.session = client.login()
            self.connected = True
        except daap.DAAPError:
            self.auth = True
            self.connected = False
            raise daap.DAAPError

    def disconnect(self):
        """
            Disconnect, clean up.
        """
        try:
            self.session.logout()
        except:
            pass
        self.session = None
        self.tracks = None
        self.database = None
        self.all = library.TrackData()
        self.connected = False

    def reload(self):
        """
            Reload the tracks from the server
        """
        APP.status.set_first(_("Retrieving track list from server..."))
        self.tracks = None
        self.database = None
        self.all = library.TrackData()
        self.get_database()
        self.convert_list()
        APP.status.clear()


    def get_database(self):
        """
            Get a DAAP database and its track list.
        """
        if self.session:
            self.database = self.session.library()
            self.get_tracks(1)

    def get_tracks(self, reset = 0):
        """
            Get the track list from a DAAP database
        """
        if reset or self.tracks == None:
            self.tracks = self.database.tracks()
            return self.tracks

    def convert_list(self):
        """
            Converts the DAAP track database into DaapTracks.
        """
        i = 0
        while i < len(self.tracks):
            tr = self.tracks[i]
            if tr:
                temp = media.Track()
                # Convert DAAPTrack's attributes to media.Track's.
                eqiv = {'title':'minm','artist':'asar','album':'asal',
                    'genre':'asgn','track':'astn','enc':'asfm',
                    'bitrate':'asbr'}
                for field in eqiv.keys():
                    try:
                        setattr(temp, field, u'%s'%tr.atom.getAtom(eqiv[field]))
                        if getattr(temp, field) == "None":
                            setattr(temp, field, "Unknown")
                    except:
                        setattr(temp, field, "Unknown")

                #TODO: convert year (asyr) here as well, what's the formula?
                try:
                    setattr(temp, '_len', tr.atom.getAtom('astm') / 1000)
                except:
                    setattr(temp, '_len', 0)
                temp.type = getattr(tr, 'type')
                temp.daapid = getattr(tr, 'id')
                temp.connection = self
    #http://<server>:<port>/databases/<dbid>/items/<id>.<type>?session-id=<sessionid>
                temp.loc = "http://%s:%s/databases/%s/items/%s.%s?session-id=%s" % \
                    (self.server, self.port, self.database.id, temp.daapid,
                    temp.type, self.session.sessionid)
                temp._rating = 2 #bad, but it works.
                self.all.append(temp)
            i = i + 1

        APP.plugin_tracks[plugins.name(__file__)] = self.all

    #@common.threaded
    def get_track(self, track_id, filename):
        """
            Save the track with track_id to filename
        """
        for t in self.tracks:
            if t.id == track_id:
                try:
                    t.save(filename)
                except CannotSendRequest:
                    dialog = gtk.MessageDialog(APP.window,
                        gtk.DIALOG_MODAL, gtk.MESSAGE_INFO, gtk.BUTTONS_OK,
                        _("""This server does not support multiple connections.
You must stop playback before downloading songs."""))




class NetworkPanel(collection.CollectionPanel):
    """
        A panel that displays the available DAAP shares and their contents.
    """
    name = 'network'
    def __init__(self, exaile, xml):
        """
            Expects the main exaile object, and a glade xml object.
        """
        self.xml = xml
        self.exaile = exaile

        self.db = exaile.db
        self.connected = False
        self.all = {}

        self.connection_list = []

        self.transfer_queue = None
        self.transferring = False
        self.queue = None
        self.model = gtk.TreeStore(gtk.gdk.Pixbuf, object, str)
        self.model_blank = gtk.TreeStore(gtk.gdk.Pixbuf, object, str)

        self.keyword = None
        self.track_cache = dict()
        self.start_count = 0
        self.tree = None
        self.connect_id = None
        self.separator_image = self.exaile.window.render_icon('gtk-remove', 
            gtk.ICON_SIZE_SMALL_TOOLBAR)
        self.artist_image = gtk.gdk.pixbuf_new_from_file(
            xl.path.get_data('images', 'artist.png'))
        self.album_image = self.exaile.window.render_icon('gtk-cdrom',
            gtk.ICON_SIZE_SMALL_TOOLBAR)
        self.track_image = gtk.gdk.pixbuf_new_from_file(
            xl.path.get_data('images', 'track.png'))
        self.genre_image = gtk.gdk.pixbuf_new_from_file(
            xl.path.get_data('images', 'genre.png'))
        self.year_image = gtk.gdk.pixbuf_new_from_file(xl.path.get_data(
            'images', 'year.png'))

        self.setup_widgets()

        self.xml.get_widget('shares_combo_box').connect('changed',
            self.switch_share)

    def switch_share(self, widget=None):
        """
            Change the active share
        """
        shares_box = self.xml.get_widget('shares_combo_box')
        if shares_box.get_active() != -1:
            if shares_box.get_active_text() == _('Custom location...'):
                while 1:
                    dialog = common.TextEntryDialog(self.exaile.window,
                        _("Enter IP address and port for share"),
                        _("Enter IP address and port."))
                    resp = dialog.run()
                    if resp == gtk.RESPONSE_OK:
                        loc = dialog.get_value()
                        address = loc.split(':')[0]
                        try:
                            port = loc.split(':')[1]
                        except IndexError:
                            dialog = gtk.MessageDialog(APP.window,
                            gtk.DIALOG_MODAL, gtk.MESSAGE_INFO, gtk.BUTTONS_OK,
                            _("""You must supply an IP address and port number, in the format
<ip address>:<port>"""))
                            dialog.run()
                            dialog.destroy()
                            continue
                        self.active_share = DaapConnection("Custom: ", address, port)
                        break
                    elif resp == gtk.RESPONSE_CANCEL:
                        shares_box.set_active(-1)
                        self.active_share = None
                        break
            else:
                self.active_share = self.connection_list[shares_box.get_active()]

            if self.active_share:
                if not self.active_share.connected:
                    try:
                        self.active_share.connect()
                    except daap.DAAPError:
                        while 1:
                            dialog = common.TextEntryDialog(self.exaile.window,
                                _("Enter password for %s") % self.active_share.name,
                                _("Password required."))
                            dialog.entry.set_visibility(False)
                            resp = dialog.run()
                            if resp == gtk.RESPONSE_OK:
                                password = dialog.get_value()
                                try:
                                    self.active_share.connect(password)
                                    break
                                except daap.DAAPError:
                                    continue
                            elif resp == gtk.RESPONSE_CANCEL:
                                shares_box.set_active(-1)
                                self.active_share = None
                                break
                    except:
                        dialog = gtk.MessageDialog(APP.window,
                            gtk.DIALOG_MODAL, gtk.MESSAGE_INFO, gtk.BUTTONS_OK,
                            _("Could not connect to server. Did you enter the correct address and port?"))
                        result = dialog.run()
                        dialog.destroy()
                        print result
                        shares_box.set_active(-1)
                        self.active_share = None

                if self.active_share and self.active_share.all:
                    self.all = self.active_share.all
                else:
                    self.all = {}
            else:
                self.all = {}
        else:
            self.all = {}
        self.load_tree(1)

    def search_tracks(self, keyword, all):
        """
            Search the tracks for keyword.
        """
        if keyword:
            check = []
            for track in self.all:
                for item in ('artist', 'album', 'title'):
                    attr = getattr(track, item)
                    if attr.lower().find(keyword.lower()) > -1:
                        check.append(track)
                        break
        else:
            check = self.all

        def stripit(field):
            return library.lstrip_special(library.the_cutter(field))

        slstrip = library.lstrip_special

        if self.choice.get_active() == 2:
            n = 5
            new = [(slstrip(a.genre), stripit(a.artist), slstrip(a.album),
                    a.track, slstrip(a.title), a) for a in check]
        elif self.choice.get_active() == 0:
            n = 4
            new = [(stripit(a.artist), slstrip(a.album), a.track, 
                    slstrip(a.title), a) for a in check]
        elif self.choice.get_active() == 1:
            n = 3
            new = [(slstrip(a.album), a.track, 
                    slstrip(a.title), a ) for a in check]

        new.sort()
        return library.TrackData([a[n] for a in new])

    def update_connections(self):
        sbox = self.xml.get_widget('shares_combo_box')
        n = 0
        while n < 100: #not good, how can we find the # of entries?
            sbox.remove_text(0) #No exception when nothing to remove?!
            n = n + 1
        self.connection_list = []
        for c in CONNECTIONS.values():
            self.connection_list.append(c)
        for c in self.connection_list:
            sbox.append_text('%s (%s:%s)' % (c.name, c.server, c.port))
        sbox.append_text(_('Custom location...'))

    def create_popup(self):
        """
            Creates the context menu.
        """
        menu = xlmisc.Menu()
        self.append = menu.append(_("Append to Current"),
            self.append_to_playlist)

        self.queue_item = menu.append(_("Queue Items"),
            self.append_to_playlist)
        menu.append_separator()

        self.save_item = menu.append(_("Save Items"),
            self.save_selected)

        self.menu = menu

    @common.threaded
    def load_tree(self, event=None):
        if event and not isinstance(event, gtk.ComboBox):
            shares_box = self.xml.get_widget('shares_combo_box')
            if shares_box.get_active() != -1:
                #active_share = self.connection_list[shares_box.get_active()]
                self.active_share.reload()
                if self.active_share.all:
                    self.all = self.active_share.all
                else:
                    self.all = {}
        collection.CollectionPanel.load_tree(self, event)

    def save_selected(self, widget=None, event=None):
        """
            Save the selected tracks to disk.
        """
        items = self.get_selected_items()
        dialog = gtk.FileChooserDialog(_("Select a save location"),
            APP.window, gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER,
            (_('Open'), gtk.RESPONSE_OK, _('Cancel'), gtk.RESPONSE_CANCEL))
        dialog.set_current_folder(APP.get_last_dir())
        dialog.set_select_multiple(False)
        result = dialog.run()
        dialog.hide()

        if result == gtk.RESPONSE_OK:
            folder = dialog.get_current_folder()
            self.save_items(items, folder)

    @common.threaded
    def save_items(self, items, folder):
        for i in items:
            tnum = i.get_track()
            if tnum < 10: tnum = "0%s"%tnum
            else: tnum = str(tnum)
            filename = "%s%s%s - %s.%s"%(folder, os.sep, tnum,
                i.get_title(), i.type)
            i.connection.get_track(i.daapid, filename)
#                print "DAAP: saving track %s to %s."%(i.daapid, filename)

def initialize():
    """
        Adds 'Network' tab to side panel, sets up Avahi.
    """
    global APP, TAB_PANE, GLADE_XML_STRING, PANEL, CONNECTIONS

    if not DAAP:
        raise plugins.PluginInitException("python-daap is not available, "
                    "disabling Music Sharing plugin.")
        return False

    if not AVAHI:
        raise plugins.PluginInitException("Avahi is not available, "
                    "disabling Music Sharing plugin.")
        return False

    xml = gtk.glade.xml_new_from_buffer(GLADE_XML_STRING,
        len(GLADE_XML_STRING))

    TAB_PANE = xml.get_widget('network_box')
    PANEL = NetworkPanel(APP, xml)
    AVAHI_INTERFACE = DaapAvahiInterface(PANEL)

    notebook = APP.xml.get_widget('side_notebook')
    tab_label = gtk.Label()
    tab_label.set_text(_('Network'))
    tab_label.set_angle(90)
    notebook.append_page(TAB_PANE, tab_label)

    TAB_PANE.show_all()
    PANEL.load_tree()
    PANEL.update_connections()

    return True


def destroy():
    """
        Removes 'Network' tab, disconnects from shares.
    """
    global CONNECTIONS
    TAB_PANE.destroy()
    AVAHI_INTERFACE = None
    for s in CONNECTIONS.values():
        #disconnect required for servers that limit connections, eg. itunes.
        s.disconnect()
#        print "DAAP: Disconnected from %s." % s.name
    CONNECTIONS = {}
    PANEL = None
    if APP.plugin_tracks.has_key(plugins.name(__file__)):
        del APP.plugin_tracks[plugins.name(__file__)]



# Copyright (C) 2007 Aren Olson
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 1, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

import dbus, time, gtk
from gettext import gettext as _
import xl.plugins as plugins
from xl import common

PLUGIN_NAME = _("AWN")
PLUGIN_AUTHORS = ['Aren Olson <reacocard@gmail.com>', 'Bryan Forbes <bryan@reigndropsfall.net>', 'Alberto Pagliarini <batopa@gmail.com>']
PLUGIN_VERSION = '0.7.4'
PLUGIN_DESCRIPTION = _(r"""Displays the current album's cover art, progress and/or remaining time in AWN.""")

PLUGIN_ENABLED = False
PLUGIN_ICON = None

SHOW_PROGRESS = False
SHOW_TIMER = False
SHOW_COVER = True

UPDATER = None
CONS = plugins.SignalContainer()

class updater:
    def __init__(self):
        """
            Set everything up.
        """
        #self.setup_awn()
        self.cover = None
        self.cover_update = 0
        self.updating = False
        self.connected = False
        self.do_update = 0

    def update_cover(self, widget=None, args=None):
        self.cover_update = 3

    def run_update(self, widget=None, args=None):
        """
            Run the updater, if necessary.
        """
        #progressbar sends two events/sec, so only update 1/2 the time.
        if self.do_update:
            self.update()
            self.do_update = 0
        else:
            self.do_update = 1

    @common.threaded
    def update(self):
        """
            Update cover, progress, time.
        """
        if not self.connected:
            self.setup_awn()
        if not self.updating:
            self.updating = True
            try:
                #Set album cover
                newcover = APP.cover.loc
                if SHOW_COVER and self.cover_update:
                    if 'nocover' in newcover:
                        self.awn.UnsetTaskIconByName(APP.window.get_title())
                        self.cover = None
                    else:
                        self.awn.SetTaskIconByName(APP.window.get_title(), newcover)
                        self.cover = newcover
                    self.cover_update = self.cover_update - 1

                #Update progress meter.
                if SHOW_PROGRESS:
                    if APP.player.is_playing():
                        position = int(round(APP.player.get_current_position(),0))
                        self.awn.SetProgressByName(APP.window.get_title(), position)
                    else:
                        self.awn.SetProgressByName(APP.window.get_title(), 100)
                else:
                    self.awn.SetProgressByName(APP.window.get_title(), 100)

                #Update remaining time.
                if SHOW_TIMER:
                    if APP.player.is_playing() or APP.player.is_paused():
                        time_remaining = APP.new_progressbar.get_text().split('/ ')[1]
                        self.awn.SetInfoByName(APP.window.get_title(), time_remaining)
                    else:
                        self.awn.UnsetInfoByName(APP.window.get_title())
                else:
                    self.awn.UnsetInfoByName(APP.window.get_title())

            except:
                self.connected = False
            self.updating = False

    def setup_awn(self):
        try:
            bus_obj = dbus.SessionBus().get_object("com.google.code.Awn",
                "/com/google/code/Awn")
            self.awn = dbus.Interface(bus_obj, "com.google.code.Awn")
            self.connected = True
        except:
            self.connected = False
            self.awn = None

def configure():
    """
        Show a configuration dialog.
    """
    global SHOW_PROGRESS, SHOW_TIMER
    dialog = plugins.PluginConfigDialog(APP.window, PLUGIN_NAME)

    box = dialog.main

    show_timer_box = gtk.RadioButton(label=_("Show song's remaining time in AWN"))
    show_timer_box.set_active(SHOW_TIMER)

    show_progress_box = gtk.RadioButton(group=show_timer_box,
        label=_("Show song's progress in AWN"))
    show_progress_box.set_active(SHOW_PROGRESS)

    do_nothing_box = gtk.RadioButton(group=show_timer_box,
        label=_("Don't show any extra information"))
    if not SHOW_PROGRESS and not SHOW_TIMER:
        do_nothing_box.set_active(True)

    box.pack_start(show_progress_box)
    box.pack_start(show_timer_box)
    box.pack_start(do_nothing_box)
    dialog.show_all()
    result = dialog.run()
    dialog.hide()

    APP.settings.set_boolean('show_progress', show_progress_box.get_active(),
        plugin=plugins.name(__file__))
    APP.settings.set_boolean('show_timer', show_timer_box.get_active(),
        plugin=plugins.name(__file__))

    SHOW_PROGRESS = APP.settings.get_boolean('show_progress',
        plugin=plugins.name(__file__), default=False)
    SHOW_TIMER = APP.settings.get_boolean('show_timer',
        plugin=plugins.name(__file__), default=False)

def initialize():
    """
        Set up the updater.
    """
    global SHOW_PROGRESS, SHOW_TIMER, UPDATER

    SHOW_PROGRESS = APP.settings.get_boolean('show_progress',
        plugin=plugins.name(__file__), default=False)

    SHOW_TIMER = APP.settings.get_boolean('show_timer',
        plugin=plugins.name(__file__), default=False)

    UPDATER = updater()

    #update whenever something changes
    CONS.connect(APP.new_progressbar, 'notify', UPDATER.run_update)
    CONS.connect(APP.cover, 'image-changed', UPDATER.update_cover)
    try:
        CONS.connect(APP.tray_icon, 'toggle-hide', UPDATER.update_cover)
    except:
        pass

    UPDATER.update_cover()

    return True

def destroy():
    """
        Disconnect from AWN.
    """
    global UPDATER
    
    CONS.disconnect_all()
    try:
        UPDATER.awn.UnsetTaskIconByName(APP.window.get_title())
        UPDATER.awn.SetProgressByName(APP.window.get_title(), 100)
        UPDATER.awn.UnsetInfoByName(APP.window.get_title())
    except:
        pass
    UPDATER.awn = None
    UPDATER = None
    
